#!/usr/bin/env python

"""
Convert JEFF 3.3 ACE data distributed by OECD/NEA into an HDF5 library that can
be used by OpenMC. It will download archives containing all the ACE files,
extract them, convert them, and write HDF5 files into a destination directory.
"""

import argparse
import sys
from pathlib import Path
from urllib.parse import urljoin

import openmc.data

from openmc_data import download, extract, state_download_size, all_release_details


# Make sure Python version is sufficient
assert sys.version_info >= (3, 6), "Python 3.6+ is required"


class CustomFormatter(
    argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter
):
    pass


parser = argparse.ArgumentParser(description=__doc__, formatter_class=CustomFormatter)
parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default=Path("jeff-3.3-hdf5"),
    help="Directory to create new library in",
)
parser.add_argument(
    "--download", action="store_true", help="Download tarball from OECD-NEA"
)
parser.add_argument(
    "--no-download",
    dest="download",
    action="store_false",
    help="Do not download tarball from OECD-NEA",
)
parser.add_argument("--extract", action="store_true", help="Extract zip files")
parser.add_argument(
    "--no-extract",
    dest="extract",
    action="store_false",
    help="Do not extract .tgz file if it has already been extracted",
)
parser.add_argument(
    "--libver",
    choices=["earliest", "latest"],
    default="earliest",
    help="Output HDF5 versioning. Use "
    "'earliest' for backwards compatibility or 'latest' for "
    "performance",
)
parser.add_argument(
    "-r",
    "--release",
    choices=["3.3"],
    default="3.3",
    help="The nuclear data library release version. "
    "The only currently supported option is 3.3.",
)
parser.add_argument(
    "-t",
    "--temperatures",
    choices=[
        "293",
        "600",
        "900",
        "1200",
        "1500",
        "1800",
    ],
    default=[
        "293",
        "600",
        "900",
        "1200",
        "1500",
        "1800",
    ],
    help="Temperatures to download in Kelvin",
    nargs="+",
)
parser.add_argument(
    "--cleanup",
    action="store_true",
    help="Remove download directories when data has " "been processed",
)
parser.add_argument(
    "--no-cleanup",
    dest="cleanup",
    action="store_false",
    help="Do not remove download directories when data has " "been processed",
)
parser.set_defaults(download=True, extract=True, cleanup=False)
args = parser.parse_args()


def key(p):
    """Return (temperature, atomic number, mass number, metastable)"""
    z, x, a, temp = p.stem.split("-")
    return int(temp), int(z), int(a[:-1]), a[-1]


def main():

    library_name = "jeff"

    cwd = Path.cwd()

    ace_files_dir = cwd.joinpath("-".join([library_name, args.release, "ace"]))
    download_path = cwd.joinpath("-".join([library_name, args.release, "download"]))

    # This dictionary contains all the unique information about each release.
    # This can be extended to accommodate new releases
    details = all_release_details[library_name][args.release]['neutron']

    # ==============================================================================
    # DOWNLOAD FILES FROM WEBSITE

    if args.download:
        state_download_size(details["compressed_file_size"], details["uncompressed_file_size"], 'GB')
        for f, t in zip(details["compressed_files"], details["temperatures"]):
            if t in args.temperatures or t is None:
                download(urljoin(details["base_url"], f), output_path=download_path)

    # ==============================================================================
    # EXTRACT FILES FROM TGZ

    if args.extract:
        extract(
            compressed_files=[
                download_path / f
                for f, t in zip(details["compressed_files"], details["temperatures"])
                if t in args.temperatures or t is None
            ],
            extraction_dir=ace_files_dir,
            del_compressed_file=args.cleanup,
        )

    # ==============================================================================
    # CONVERT INCIDENT NEUTRON FILES

    # Create output directory if it doesn't exist
    args.destination.mkdir(parents=True, exist_ok=True)

    lib = openmc.data.DataLibrary()

    for p in sorted(ace_files_dir.glob(details['neutron_files']), key=key):
        print(f"Converting: {p}")
        temp, z, a, m = key(p)

        data = openmc.data.IncidentNeutron.from_ace(p)
        if m == "m" and not data.name.endswith("_m1"):
            # Correct metastable
            data.metastable = 1
            data.name += "_m1"

        for T in list(filter(None, args.temperatures)):
            p_add = ace_files_dir / f"ace_{T}" / (p.stem.replace("293", T) + ".ace")
            print(f"Adding temperature: {p_add}")
            data.add_temperature_from_ace(p_add)

        h5_file = args.destination / f"{data.name}.h5"
        data.export_to_hdf5(h5_file, "w", libver=args.libver)
        lib.register_file(h5_file)

    # ==============================================================================
    # CONVERT THERMAL SCATTERING FILES

    thermal_mats = [
        "al-sap",
        "be",
        "ca-cah2",
        "d-d2o",
        "graph",
        "h-cah2",
        "h-ch2",
        "h-h2o",
        "h-ice",
        "h-zrh",
        "mesi",
        "mg",
        "o-d2o",
        "orto-d",
        "orto-h",
        "o-sap",
        "para-d",
        "para-h",
        "sili",
        "tolu",
    ]

    def thermal_temp(p):
        return int(p.stem.split("-")[-1])

    thermal_dir = ace_files_dir / details["thermal_files"]

    for mat in thermal_mats:
        for i, p in enumerate(
            sorted(thermal_dir.glob(f"{mat}*.ace"), key=thermal_temp)
        ):
            if i == 0:
                print(f"Converting: {p}")
                data = openmc.data.ThermalScattering.from_ace(p)
            else:
                print(f"Adding temperature: {p}")
                data.add_temperature_from_ace(p)

        h5_file = args.destination / f"{data.name}.h5"
        data.export_to_hdf5(h5_file, "w", libver=args.libver)
        lib.register_file(h5_file)

    lib.export_to_xml(args.destination / "cross_sections.xml")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import os
from argparse import ArgumentParser
from pathlib import Path
from zipfile import ZipFile

import openmc.deplete

from openmc_data.utils import download


URLS = [
    'https://www.nndc.bnl.gov/endf-b7.1/zips/ENDF-B-VII.1-neutrons.zip',
    'https://www.nndc.bnl.gov/endf-b7.1/zips/ENDF-B-VII.1-decay.zip',
    'https://www.nndc.bnl.gov/endf-b7.1/zips/ENDF-B-VII.1-nfy.zip'
]

# Parse command line arguments
parser = ArgumentParser()
parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default='chain_endfb71.xml',
    help="filename of the chain file xml file produced.",
)
args = parser.parse_args()


def main():
    endf_dir = os.environ.get("OPENMC_ENDF_DATA")
    if endf_dir is not None:
        endf_dir = Path(endf_dir)
    elif all(os.path.isdir(lib) for lib in ("neutrons", "decay", "nfy")):
        endf_dir = Path(".")
    else:
        for url in URLS:
            basename = download(url)
            with ZipFile(basename, 'r') as zf:
                print('Extracting {}...'.format(basename))
                zf.extractall()
        endf_dir = Path(".")

    decay_files = list((endf_dir / "decay").glob("*endf"))
    neutron_files = list((endf_dir / "neutrons").glob("*endf"))
    nfy_files = list((endf_dir / "nfy").glob("*endf"))

    # Remove erroneous Be7 evaluation that can cause problems
    decay_files.remove(endf_dir / "decay" / "dec-004_Be_007.endf")
    neutron_files.remove(endf_dir / "neutrons" / "n-004_Be_007.endf")

    # check files exist
    for flist, ftype in [(decay_files, "decay"), (neutron_files, "neutron"),
                         (nfy_files, "neutron fission product yield")]:
        if not flist:
            raise IOError("No {} endf files found in {}".format(ftype, endf_dir))

    chain = openmc.deplete.Chain.from_endf(decay_files, nfy_files, neutron_files)

    chain.export_to_xml(args.destination)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""
Download FENDL-3.1d, FENDL-3.1a, FENDL-3.0 or FENDL-2.1 ACE data from the IAEA
and convert it to a HDF5 library for use with OpenMC.
"""

import argparse
import ssl
import subprocess
import warnings
from pathlib import Path
from shutil import rmtree
from textwrap import dedent
from urllib.parse import urljoin

import openmc.data
from openmc_data import download, state_download_size, all_release_details


class CustomFormatter(
    argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter
):
    pass


parser = argparse.ArgumentParser(description=__doc__, formatter_class=CustomFormatter)
parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default=None,
    help="Directory to create new library in",
)
parser.add_argument(
    "--download", action="store_true", help="Download files from IAEA-NDS"
)
parser.add_argument(
    "--no-download",
    dest="download",
    action="store_false",
    help="Do not download files from IAEA-NDS",
)
parser.add_argument("--extract", action="store_true", help="Extract tar/zip files")
parser.add_argument(
    "--no-extract",
    dest="extract",
    action="store_false",
    help="Do not extract tar/zip files",
)
parser.add_argument(
    "--libver",
    choices=["earliest", "latest"],
    default="earliest",
    help="Output HDF5 versioning. Use "
    "'earliest' for backwards compatibility or 'latest' for "
    "performance",
)
parser.add_argument(
    "-r",
    "--release",
    choices=["3.2", "3.1d", "3.1a", "3.0", "2.1"],
    default="3.2",
    help="The nuclear data library "
    "release version. The currently supported options are "
    "3.2, 3.1d, 3.1a, 3.0 and 2.1",
)
parser.add_argument(
    "-p",
    "--particles",
    choices=["neutron", "photon"],
    nargs="+",
    default=["neutron", "photon"],
    help="Incident particles to include",
)
parser.add_argument(
    "--cleanup",
    action="store_true",
    help="Remove download directories when data has " "been processed",
)
parser.add_argument(
    "--no-cleanup",
    dest="cleanup",
    action="store_false",
    help="Do not remove download directories when data has " "been processed",
)
parser.set_defaults(download=True, extract=True, cleanup=False)
args = parser.parse_args()


# =============================================================================
# FUNCTIONS FOR DEALING WITH SPECIAL CASES
#
# Each of these functions should take a Path object which points to the file
# The function should return a bool which determines whether the file should be
# ignored.


def fendl30_k39(file_path):
    """Function to check for k-39 error in FENDL-3.0"""
    if "Inf" in open(file_path, "r").read():
        ace_error_warning = """
        {} contains 'Inf' values within the XSS array
        which prevent conversion to a HDF5 file format. This is a known issue
        in FENDL-3.0. {} has not been added to the cross section library.
        """.format(
            file_path, file_path.name
        )
        err_msg = dedent(ace_error_warning)
        return {"skip_file": True, "err_msg": err_msg}
    else:
        return {"skip_file": False}


def check_special_case(particle_details, script_step):
    """
    Helper function for checking if there are any special cases defined:
    Returns the special Cases relevant to a specific part of the script.
    If there are no special cases, return an empty dict
    """
    if "special_cases" in particle_details:
        if script_step in particle_details["special_cases"]:
            return particle_details["special_cases"][script_step]
    return {}


def main():

    library_name = "fendl"
    cwd = Path.cwd()

    ace_files_dir = cwd.joinpath("-".join([library_name, args.release, "ace"]))
    endf_files_dir = cwd.joinpath("-".join([library_name, args.release, "endf"]))

    download_path = cwd.joinpath("-".join([library_name, args.release, "download"]))
    # the destination is decided after the release is know to avoid putting
    # the release in a folder with a misleading name
    if args.destination is None:
        args.destination = Path("-".join([library_name, args.release, "hdf5"]))

    # This dictionary contains all the unique information about each release.
    # This can be extended to accommodate new releases
    release_details = all_release_details[library_name]

    # todo refactor this into the release dictionary
    if args.release == "3.0":
        release_details[args.release]["neutron"]["special_cases"] = {
            "process": {"19K_039.ace": fendl30_k39}
        }

    compressed_file_size = uncompressed_file_size = 0
    for p in ("neutron", "photon"):
        if p in args.particles:
            compressed_file_size += release_details[args.release][p][
                "compressed_file_size"
            ]
            uncompressed_file_size += release_details[args.release][p][
                "uncompressed_file_size"
            ]


    # Warnings to be printed at the end of the script.
    output_warnings = []

    # ==============================================================================
    # DOWNLOAD FILES FROM IAEA SITE

    if args.download:
        state_download_size(compressed_file_size, uncompressed_file_size, 'MB')

        for particle in args.particles:
            # Create a directory to hold the downloads
            particle_download_path = download_path / particle

            particle_details = release_details[args.release][particle]
            for f in particle_details["compressed_files"]:
                download(
                    urljoin(particle_details["base_url"], f),
                    as_browser=True,
                    context=ssl._create_unverified_context(),
                    output_path=particle_download_path,
                )

    # ==============================================================================
    # EXTRACT FILES FROM ZIP
    if args.extract:
        for particle in args.particles:

            particle_details = release_details[args.release][particle]

            special_cases = check_special_case(particle_details, "extract")

            if particle_details["file_type"] == "ace":
                extraction_dir = ace_files_dir
            elif particle_details["file_type"] == "endf":
                extraction_dir = endf_files_dir

            for f in particle_details["compressed_files"]:
                # Check if file requires special handling
                if f in special_cases:
                    ret = special_cases[f](Path(f))
                    if "err_msg" in ret:
                        output_warnings.append(ret["err_msg"])
                    if ret["skip_file"]:
                        continue

                # Extract files, the fendl release was compressed using type 9 zip format
                # unfortunatly which is incompatible with the standard python zipfile library
                # therefore the following system command is used
                subprocess.call(
                    ["unzip", "-o", download_path / particle / f, "-d", extraction_dir]
                )

        if args.cleanup and download_path.exists():
            rmtree(download_path)

    # ==============================================================================
    # GENERATE HDF5 LIBRARY

    library = openmc.data.DataLibrary()

    for particle in args.particles:
        # Create output directories if it doesn't exist
        particle_destination = args.destination / particle
        particle_destination.mkdir(parents=True, exist_ok=True)

        particle_details = release_details[args.release][particle]

        # Get dictionary of special cases for particle
        special_cases = check_special_case(particle_details, "process")

        if particle == "neutron":
            # Get a list of all ACE files
            neutron_files = ace_files_dir.glob(
                release_details[args.release]["neutron"]["ace_files"]
            )

            # excluding files ending with _ that are
            # old incorrect files kept in the release for backwards compatability
            neutron_files = [
                f
                for f in neutron_files
                if not f.name.endswith("_") and not f.name.endswith(".xsd")
            ]

            for filename in sorted(neutron_files):
                # Handling for special cases
                if filename.name in special_cases:
                    ret = special_cases[filename.name](filename)
                    if "err_msg" in ret:
                        output_warnings.append(ret["err_msg"])
                    if ret["skip_file"]:
                        continue

                print(f"Converting: {filename}")
                data = openmc.data.IncidentNeutron.from_ace(filename)

                # Export HDF5 file
                h5_file = particle_destination / f"{data.name}.h5"
                print(f"Writing {h5_file}...")
                data.export_to_hdf5(h5_file, "w", libver=args.libver)

                # Register with library
                library.register_file(h5_file)

            # Remove the ace files if required
            if args.cleanup and ace_files_dir.exists():
                rmtree(ace_files_dir)

        elif particle == "photon":

            photon_files = endf_files_dir.glob(
                release_details[args.release]["photon"]["endf_files"]
            )

            for photo_path in sorted(photon_files):

                # Check if file requires special handling
                if photo_path.name in special_cases:
                    ret = special_cases[photo_path.name](photo_path)
                    if "err_msg" in ret:
                        output_warnings.append(ret["err_msg"])
                    if ret["skip_file"]:
                        continue

                print(f"Converting: {photo_path}")
                evaluations = openmc.data.endf.get_evaluations(photo_path)
                for ev in evaluations:
                    # Export HDF5 file
                    data = openmc.data.IncidentPhoton.from_endf(ev)
                    h5_file = particle_destination / f"{data.name}.h5"
                    print(f"Writing {h5_file}...")
                    data.export_to_hdf5(h5_file, "w", libver=args.libver)

                # Register with library
                library.register_file(h5_file)

            # Remove the ENDF files if required
            if args.cleanup and endf_files_dir.exists():
                rmtree(endf_files_dir)

    # Write cross_sections.xml
    print("Writing ", args.destination / "cross_sections.xml")
    library.export_to_xml(args.destination / "cross_sections.xml")

    # Print any warnings
    for warning in output_warnings:
        warnings.warn(warning)


if __name__ == "__main__":
    main()



all_chain_release_details = {
    "nndc": {
        "b7.1": {
            "chain": {
                "url": "https://raw.githubusercontent.com/openmc-data-storage/openmc_data/849229a66060240a09b45236f3d8d43f0434235b/openmc_data/depletion/chain-nndc-b7.1.xml",
            }
        },
    }
}



#!/usr/bin/env python3

"""
Download TENDL 2019/2017/2015 ACE files from PSI and
convert them to HDF5 libraries for use with OpenMC.
"""

import argparse
import sys
from pathlib import Path
from urllib.parse import urljoin

import openmc.data
from openmc_data import download, extract, state_download_size, all_release_details


# Make sure Python version is sufficient
assert sys.version_info >= (3, 6), "Python 3.6+ is required"


class CustomFormatter(
    argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter
):
    pass


parser = argparse.ArgumentParser(description=__doc__, formatter_class=CustomFormatter)
parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default=None,
    help="Directory to create new library in",
)
parser.add_argument("--download", action="store_true", help="Download files from PSI")
parser.add_argument(
    "--no-download",
    dest="download",
    action="store_false",
    help="Do not download files from PSI",
)
parser.add_argument("--extract", action="store_true", help="Extract tar/zip files")
parser.add_argument(
    "--no-extract",
    dest="extract",
    action="store_false",
    help="Do not extract tar/zip files",
)
parser.add_argument(
    "--libver",
    choices=["earliest", "latest"],
    default="latest",
    help="Output HDF5 versioning. Use "
    "'earliest' for backwards compatibility or 'latest' for "
    "performance",
)
parser.add_argument(
    "-r",
    "--release",
    choices=["2015", "2017", "2019", "2021"],
    default="2021",
    help="The nuclear data library release "
    "version. The currently supported options are 2015, "
    "2017, 2019, and 2021.",
)
parser.add_argument(
    "--cleanup",
    action="store_true",
    help="Remove download directories when data has " "been processed",
)
parser.add_argument(
    "--no-cleanup",
    dest="cleanup",
    action="store_false",
    help="Do not remove download directories when data has " "been processed",
)
parser.set_defaults(download=True, extract=True, cleanup=False)
args = parser.parse_args()


def main():

    library_name = "tendl"

    cwd = Path.cwd()

    ace_files_dir = cwd.joinpath("-".join([library_name, args.release, "ace"]))
    download_path = cwd.joinpath("-".join([library_name, args.release, "download"]))
    # the destination is decided after the release is known
    # to avoid putting the release in a folder with a misleading name
    if args.destination is None:
        args.destination = Path("-".join([library_name, args.release, "hdf5"]))

    # This dictionary contains all the unique information about each release.
    # This can be extended to accommodated new releases
    release_details = all_release_details[library_name][args.release]["neutron"]

    # ==============================================================================
    # DOWNLOAD FILES FROM WEBSITE

    if args.download:
        state_download_size(
            release_details['compressed_file_size'],
            release_details['uncompressed_file_size'],
            'GB'
        )
        for f in release_details["compressed_files"]:
            # Establish connection to URL
            download(urljoin(release_details["base_url"], f), output_path=download_path)

    # ==============================================================================
    # EXTRACT FILES FROM TGZ

    if args.extract:
        extract(
            compressed_files=[
                download_path / f for f in release_details["compressed_files"]
            ],
            extraction_dir=ace_files_dir,
            del_compressed_file=args.cleanup,
        )

    # ==============================================================================
    # CHANGE ZAID FOR METASTABLES

    metastables = ace_files_dir.glob(release_details["metastables"])
    for path in metastables:
        print("    Fixing {} (ensure metastable)...".format(path))
        text = open(path, "r").read()
        mass_first_digit = int(text[3])
        if mass_first_digit <= 2:
            text = text[:3] + str(mass_first_digit + 4) + text[4:]
            open(path, "w").write(text)

    # ==============================================================================
    # GENERATE HDF5 LIBRARY -- NEUTRON FILES

    # Get a list of all ACE files
    neutron_files = ace_files_dir.glob(release_details["neutron_files"])

    # Create output directory if it doesn't exist
    args.destination.mkdir(parents=True, exist_ok=True)

    library = openmc.data.DataLibrary()

    for filename in sorted(neutron_files):

        # this is a fix for the TENDL-2017 release where the B10 ACE file which has an error on one of the values
        if args.release == "2017" and filename.name == "B010":
            text = open(filename, "r").read()
            if text[423:428] == "86843":
                print("Manual fix for incorrect value in ACE file")
                # see OpenMC user group issue for more details
                text = "".join(text[:423]) + "86896" + "".join(text[428:])
                open(filename, "w").write(text)

        print(f"Converting: {filename}")
        data = openmc.data.IncidentNeutron.from_ace(filename)

        # Export HDF5 file
        h5_file = args.destination / f"{data.name}.h5"
        print("Writing {}...".format(h5_file))
        data.export_to_hdf5(h5_file, "w", libver=args.libver)

        # Register with library
        library.register_file(h5_file)

    # Write cross_sections.xml
    library.export_to_xml(args.destination / "cross_sections.xml")


if __name__ == "__main__":
    main()



from .utils import download, extract, process_neutron, process_thermal, state_download_size
from .urls import all_release_details
from .urls_h5 import all_h5_release_details
from .urls_xml import all_chain_release_details



#!/usr/bin/env python3

"""
Convert ENDF/B-VII.1 ACE data from the MCNP6 distribution into an HDF5 library
that can be used by OpenMC. This assumes that you have a directory containing
subdirectories 'endf71x' and 'ENDF71SaB'. Optionally, if a recent photoatomic
library (e.g., eprdata14) is available, it can also be converted using the
--photon argument.
"""

import argparse
from collections import defaultdict
from pathlib import Path
import sys

import openmc.data


# Make sure Python version is sufficient
assert sys.version_info >= (3, 6), "Python 3.6+ is required"



class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter,
                      argparse.RawDescriptionHelpFormatter):
    pass


parser = argparse.ArgumentParser(
    description=__doc__,
    formatter_class=CustomFormatter
)
parser.add_argument('-d', '--destination', type=Path, default=Path('mcnp_endfb71'),
                    help='Directory to create new library in')
parser.add_argument('--libver', choices=['earliest', 'latest'],
                    default='earliest', help="Output HDF5 versioning. Use "
                    "'earliest' for backwards compatibility or 'latest' for "
                    "performance")
parser.add_argument('-p', '--photon', type=Path,
                    help='Path to photoatomic data library (eprdata12 or later)')
parser.add_argument('mcnpdata', type=Path,
                    help='Directory containing endf71x and ENDF71SaB')
args = parser.parse_args()


def main():

    # Check arguments to make sure they're valid
    assert args.mcnpdata.is_dir(), 'mcnpdata argument must be a directory'
    if args.photon is not None:
        assert args.photon.is_file(), 'photon argument must be an existing file'

    # Get a list of all ACE files
    endf71x = list(args.mcnpdata.glob('endf71x/*/*.7??nc'))
    endf71sab = list(args.mcnpdata.glob('ENDF71SaB/*.??t'))

    # Check for fixed H1 files and remove old ones if present
    hydrogen = args.mcnpdata / 'endf71x' / 'H'
    if (hydrogen / '1001.720nc').is_file():
        for i in range(10, 17):
            endf71x.remove(hydrogen / f'1001.7{i}nc')

    # There's a bug in H-Zr at 1200 K
    thermal = args.mcnpdata / 'ENDF71SaB'
    endf71sab.remove(thermal / 'h-zr.27t')

    # Check for updated TSL files and remove old ones if present
    checks = [
        ('sio2', 10, range(20, 37)),
        ('u-o2', 30, range(20, 28)),
        ('zr-h', 30, range(20, 28))
    ]
    for material, good, bad in checks:
        if (thermal / f'{material}.{good}t').is_file():
            for suffix in bad:
                f = thermal / f'{material}.{suffix}t'
                if f.is_file():
                    endf71sab.remove(f)

    # Group together tables for the same nuclide
    tables = defaultdict(list)
    for p in sorted(endf71x + endf71sab):
        tables[p.stem].append(p)

    # Create output directory if it doesn't exist
    (args.destination / 'photon').mkdir(parents=True, exist_ok=True)

    library = openmc.data.DataLibrary()

    for name, paths in sorted(tables.items()):
        # Convert first temperature for the table
        p = paths[0]
        print(f'Converting: {p}')
        if p.name.endswith('t'):
            data = openmc.data.ThermalScattering.from_ace(p)
        else:
            data = openmc.data.IncidentNeutron.from_ace(p, 'mcnp')

        # For each higher temperature, add cross sections to the existing table
        for p in paths[1:]:
            print(f'Adding: {p}')
            if p.name.endswith('t'):
                data.add_temperature_from_ace(p)
            else:
                data.add_temperature_from_ace(p, 'mcnp')

        # Export HDF5 file
        h5_file = args.destination / f'{data.name}.h5'
        print(f'Writing {h5_file}...')
        data.export_to_hdf5(h5_file, 'w', libver=args.libver)

        # Register with library
        library.register_file(h5_file)

    # Handle photoatomic data
    if args.photon is not None:
        lib = openmc.data.ace.Library(args.photon)

        for table in lib.tables:
            # Convert first temperature for the table
            print(f'Converting: {table.name}')
            data = openmc.data.IncidentPhoton.from_ace(table)

            # Export HDF5 file
            h5_file = args.destination / 'photon' / f'{data.name}.h5'
            print(f'Writing {h5_file}...')
            data.export_to_hdf5(h5_file, 'w', libver=args.libver)

            # Register with library
            library.register_file(h5_file)

    # Write cross_sections.xml
    library.export_to_xml(args.destination / 'cross_sections.xml')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""
Determine Q values equivalent to the defaults in Serpent
"""

from argparse import ArgumentParser
from pathlib import Path
import json

import openmc.data

# Get command line argument
parser = ArgumentParser()
parser.add_argument('dir', type=Path, help='Directory containing ENDF neutron sublibrary files')
parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default='serpent_fissq.json',
    help="filename of the heating values file produced",
)
args = parser.parse_args()


def main():

    # Get Q value for U235
    u235 = openmc.data.IncidentNeutron.from_endf(args.dir / 'n-092_U_235.endf')
    q_u235 = u235[18].q_value

    # Fixed heating value from Serpent
    # See http://serpent.vtt.fi/mediawiki/index.php/Input_syntax_manual#set_fissh
    heat_u235 = 202.27e6

    # Get Q values for all fissionable nuclides and scale by the ratio of the U235 Q
    # value and heating value
    serpent_fission_q = {}
    for path in args.dir.glob('*.endf'):
        nuc = openmc.data.IncidentNeutron.from_endf(path)
        if nuc.fission_energy is None:
            continue
        q = nuc[18].q_value
        serpent_fission_q[nuc.name] = heat_u235 * q / q_u235

    # Write heating values to JSON file
    with open(args.destination, 'w') as f:
        json.dump(serpent_fission_q, f, indent=2)



#!/usr/bin/env python

from urllib.parse import urlencode
from urllib.request import urlopen
from lxml import html

import numpy as np
import h5py
from openmc.data import ATOMIC_SYMBOL


def main():

    base_url = 'https://physics.nist.gov/cgi-bin/Star/e_table-t.pl'
    energies = np.logspace(-3, 3, 200)
    data = {'matno': '', 'Energies': '\n'.join(str(x) for x in energies)}
    columns = {1: 's_collision', 2: 's_radiative'}

    # ==============================================================================
    # SCRAPE DATA FROM ESTAR SITE AND GENERATE STOPPING POWER HDF5 FILE

    print('Generating stopping_powers.h5...')

    with h5py.File('stopping_powers.h5', 'w') as f:

        # Write energies
        f.create_dataset('energy', data=energies)

        # Look over atomic number; ESTAR only goes up to Z=98 (Californium)
        for Z in range(1, 99):
            print('Processing {} data...'.format(ATOMIC_SYMBOL[Z]))

            # Update form-encoded data to send in POST request for this element
            data['matno'] = f'{Z:03}'
            payload = urlencode(data).encode("utf-8")

            # Retrieve data from ESTAR site
            with urlopen(url=base_url, data=payload) as response:
                r = response.read()

            # Remove text and reformat data -- omit first 12 and last 5 lines to get
            # only data in table
            r = html.fromstring(r).xpath('//pre//text()')
            values = np.fromstring(' '.join(r[12:-5]), sep=' ').reshape((-1, 5)).T

            # Create group for this element
            group = f.create_group(f'{Z:03}')

            # Write the mean excitation energy
            attributes = np.fromstring(r[3], sep=' ')
            group.attrs['I'] = attributes[2]

            # Write collision and radiative stopping powers
            for i in columns:
                group.create_dataset(columns[i], data=values[i])


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""
Download ENDF/B-VIII.0 or ENDF/B-VII.1 library for use in OpenMC by first
processing ENDF files using NJOY. The resulting library will contain incident
neutron, incident photon, and thermal scattering data.
"""


import argparse
import sys
import tarfile
import zipfile
from multiprocessing import Pool
from pathlib import Path
from shutil import rmtree, copy, copyfileobj

import openmc.data
from openmc_data import download, process_neutron, process_thermal, state_download_size

# Make sure Python version is sufficient
assert sys.version_info >= (3, 6), "Python 3.6+ is required"


class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter,
                      argparse.RawDescriptionHelpFormatter):
    pass


parser = argparse.ArgumentParser(
    description=__doc__,
    formatter_class=CustomFormatter
)

parser.add_argument('-d', '--destination', type=Path,
                    help='Directory to create new library in')
parser.add_argument('--download', action='store_true',
                    help='Download zip files from NNDC')
parser.add_argument('--no-download', dest='download', action='store_false',
                    help='Do not download zip files from NNDC')
parser.add_argument('--extract', action='store_true',
                    help='Extract zip files')
parser.add_argument('--no-extract', dest='extract', action='store_false',
                    help='Do not extract zip files')
parser.add_argument('--libver', choices=['earliest', 'latest'],
                    default='earliest', help="Output HDF5 versioning. Use "
                    "'earliest' for backwards compatibility or 'latest' for "
                    "performance")
parser.add_argument('-r', '--release', choices=['vii.1', 'viii.0'],
                    default='viii.0', help="The nuclear data library release "
                    "version. The currently supported options are vii.1, "
                    "viii.0")
parser.add_argument('-p', '--particles', choices=['neutron', 'photon', 'wmp'],
                    nargs='+', default=['neutron', 'photon'],
                    help="Incident particles to include, wmp is not available "
                    "for release b8.0 at the moment")
parser.add_argument('--cleanup', action='store_true',
                    help="Remove download directories when data has "
                    "been processed")
parser.add_argument('--no-cleanup', dest='cleanup', action='store_false',
                    help="Do not remove download directories when data has "
                    "been processed")
parser.add_argument('--temperatures', type=float,
                    default=[250.0, 293.6, 600.0, 900.0, 1200.0, 2500.0],
                    help="Temperatures in Kelvin", nargs='+')
parser.set_defaults(download=True, extract=True, cleanup=False)
args = parser.parse_args()


def sort_key(path):
    if path.name.startswith('c_'):
        # Ensure that thermal scattering gets sorted after neutron data
        return (1000, path)
    else:
        return openmc.data.zam(path.stem)


def main():

    library_name = 'endfb'

    cwd = Path.cwd()

    endf_files_dir = cwd.joinpath('-'.join([library_name, args.release, 'endf']))
    neutron_dir = endf_files_dir / 'neutron'
    thermal_dir = endf_files_dir / 'thermal_scatt'
    download_path = cwd.joinpath('-'.join([library_name, args.release, 'download']))
    # the destination is decided after the release is known
    # to avoid putting the release in a folder with a misleading name
    if args.destination is None:
        args.destination = Path('-'.join([library_name, args.release, 'hdf5']))

    # This dictionary contains all the unique information about each release. This
    # can be extended to accommodate new releases
    release_details = {
        'vii.1': {
            'neutron': {
                'base_url': 'http://www.nndc.bnl.gov/endf-b7.1/zips/',
                'compressed_files': ['ENDF-B-VII.1-neutrons.zip',
                                    'ENDF-B-VII.1-thermal_scatt.zip'],
                'checksums': ['e5d7f441fc4c92893322c24d1725e29c',
                            'fe590109dde63b2ec5dc228c7b8cab02'],
                'file_type': 'endf',
                'endf_files': neutron_dir.rglob('n-*.endf'),
                'sab_files': [
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-HinH2O.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-HinCH2.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-HinZrH.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-ortho-H.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-para-H.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-benzine.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-l-CH4.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-s-CH4.endf'),
                    (neutron_dir / 'n-001_H_002.endf', neutron_dir / 'tsl-DinD2O.endf'),
                    (neutron_dir / 'n-001_H_002.endf', neutron_dir / 'tsl-ortho-D.endf'),
                    (neutron_dir / 'n-001_H_002.endf', neutron_dir / 'tsl-para-D.endf'),
                    (neutron_dir / 'n-004_Be_009.endf', neutron_dir / 'tsl-BeinBeO.endf'),
                    (neutron_dir / 'n-004_Be_009.endf', neutron_dir / 'tsl-Be-metal.endf'),
                    (neutron_dir / 'n-006_C_000.endf', neutron_dir / 'tsl-graphite.endf'),
                    (neutron_dir / 'n-008_O_016.endf', neutron_dir / 'tsl-OinBeO.endf'),
                    (neutron_dir / 'n-008_O_016.endf', neutron_dir / 'tsl-OinUO2.endf'),
                    (neutron_dir / 'n-013_Al_027.endf', neutron_dir / 'tsl-013_Al_027.endf'),
                    (neutron_dir / 'n-026_Fe_056.endf', neutron_dir / 'tsl-026_Fe_056.endf'),
                    (neutron_dir / 'n-014_Si_028.endf', neutron_dir / 'tsl-SiO2.endf'),
                    (neutron_dir / 'n-040_Zr_090.endf', neutron_dir / 'tsl-ZrinZrH.endf'),
                    (neutron_dir / 'n-092_U_238.endf', neutron_dir / 'tsl-UinUO2.endf')
                ],
                'compressed_file_size': 226,
                'uncompressed_file_size': 916
            },
            'photon': {
                'base_url': 'http://www.nndc.bnl.gov/endf-b7.1/zips/',
                'compressed_files': ['ENDF-B-VII.1-photoat.zip',
                                    'ENDF-B-VII.1-atomic_relax.zip'],
                'checksums': ['5192f94e61f0b385cf536f448ffab4a4',
                            'fddb6035e7f2b6931e51a58fc754bd10'],
                'file_type': 'endf',
                'photo_files': endf_files_dir.joinpath('photoat').rglob('*.endf'),
                'atom_files': endf_files_dir.joinpath('atomic_relax').rglob('*.endf'),
                'compressed_file_size': 9,
                'uncompressed_file_size': 45
            },
            'wmp': {
                'base_url': 'https://github.com/mit-crpg/WMP_Library/releases/download/v1.1/',
                'compressed_files': ['WMP_Library_v1.1.tar.gz'],
                'file_type': 'wmp',
                'compressed_file_size': 12,
                'uncompressed_file_size': 17
            }
        },
        'viii.0': {
            'neutron': {
                'base_url': 'https://www.nndc.bnl.gov/endf-b8.0/',
                'compressed_files': ['zips/ENDF-B-VIII.0_neutrons.zip',
                                    'zips/ENDF-B-VIII.0_thermal_scatt.zip',
                                    'erratafiles/n-005_B_010.endf'],
                'checksums': ['90c1b1a6653a148f17cbf3c5d1171859',
                            'ecd503d3f8214f703e95e17cc947062c',
                            'eaf71eb22258f759abc205a129d8715a'],
                'file_type': 'endf',
                'endf_files': neutron_dir.rglob('n-*.endf'),
                'sab_files': [
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-HinC5O2H8.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-HinH2O.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-HinCH2.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-HinZrH.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-HinIceIh.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-HinYH2.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-ortho-H.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-para-H.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-benzene.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-l-CH4.endf'),
                    (neutron_dir / 'n-001_H_001.endf', neutron_dir / 'tsl-s-CH4.endf'),
                    (neutron_dir / 'n-001_H_002.endf', neutron_dir / 'tsl-DinD2O.endf'),
                    (neutron_dir / 'n-001_H_002.endf', neutron_dir / 'tsl-ortho-D.endf'),
                    (neutron_dir / 'n-001_H_002.endf', neutron_dir / 'tsl-para-D.endf'),
                    (neutron_dir / 'n-004_Be_009.endf', neutron_dir / 'tsl-BeinBeO.endf'),
                    (neutron_dir / 'n-004_Be_009.endf', neutron_dir / 'tsl-Be-metal.endf'),
                    (neutron_dir / 'n-006_C_012.endf', neutron_dir / 'tsl-CinSiC.endf'),
                    (neutron_dir / 'n-006_C_012.endf', neutron_dir / 'tsl-crystalline-graphite.endf'),
                    (neutron_dir / 'n-006_C_012.endf', neutron_dir / 'tsl-reactor-graphite-10P.endf'),
                    (neutron_dir / 'n-006_C_012.endf', neutron_dir / 'tsl-reactor-graphite-30P.endf'),
                    (neutron_dir / 'n-007_N_014.endf', neutron_dir / 'tsl-NinUN.endf'),
                    (neutron_dir / 'n-008_O_016.endf', neutron_dir / 'tsl-OinBeO.endf'),
                    (neutron_dir / 'n-008_O_016.endf', neutron_dir / 'tsl-OinD2O.endf'),
                    (neutron_dir / 'n-008_O_016.endf', neutron_dir / 'tsl-OinIceIh.endf'),
                    (neutron_dir / 'n-008_O_016.endf', neutron_dir / 'tsl-OinUO2.endf'),
                    (neutron_dir / 'n-013_Al_027.endf', neutron_dir / 'tsl-013_Al_027.endf'),
                    (neutron_dir / 'n-026_Fe_056.endf', neutron_dir / 'tsl-026_Fe_056.endf'),
                    (neutron_dir / 'n-014_Si_028.endf', neutron_dir / 'tsl-SiinSiC.endf'),
                    (neutron_dir / 'n-014_Si_028.endf', neutron_dir / 'tsl-SiO2-alpha.endf'),
                    (neutron_dir / 'n-014_Si_028.endf', neutron_dir / 'tsl-SiO2-beta.endf'),
                    (neutron_dir / 'n-039_Y_089.endf', neutron_dir / 'tsl-YinYH2.endf'),
                    (neutron_dir / 'n-040_Zr_090.endf', neutron_dir / 'tsl-ZrinZrH.endf'),
                    (neutron_dir / 'n-092_U_238.endf', neutron_dir / 'tsl-UinUN.endf'),
                    (neutron_dir / 'n-092_U_238.endf', neutron_dir / 'tsl-UinUO2.endf')
                ],
                'compressed_file_size': 296+59+0.849,
                'uncompressed_file_size': 999999
            },
            'photon': {
                'base_url': 'https://www.nndc.bnl.gov/endf-b8.0/',
                'compressed_files': ['zips/ENDF-B-VIII.0_photoat.zip',
                                    'erratafiles/atomic_relax.tar.gz'],
                'checksums': ['d49f5b54be278862e1ce742ccd94f5c0',
                            '805f877c59ad22dcf57a0446d266ceea'],
                'file_type': 'endf',
                'photo_files': endf_files_dir.joinpath('photoat').rglob('*.endf'),
                'atom_files': endf_files_dir.joinpath('atom').rglob('*.endf'),
                'compressed_file_size': 1.2+35,
                'uncompressed_file_size': 999999
            }
        }
    }

    compressed_file_size, uncompressed_file_size = 0, 0
    for r in args.release:
        for p in args.particles:
            compressed_file_size += release_details[args.release][p]['compressed_file_size']
            uncompressed_file_size += release_details[args.release][p]['uncompressed_file_size']

    # ==============================================================================
    # DOWNLOAD FILES FROM NNDC SITE

    if args.download:
        state_download_size(compressed_file_size, uncompressed_file_size, 'MB')
        for particle in args.particles:
            details = release_details[args.release][particle]
            for i, f in enumerate(details['compressed_files']):
                url = details['base_url'] + f
                if 'checksums' in details.keys():
                    checksum = details['checksums'][i]
                    downloaded_file = download(url,
                                            output_path=download_path / particle,
                                            checksum=checksum)
                else:
                    downloaded_file = download(url,
                                            output_path=download_path / particle,
                                            )

    # ==============================================================================
    # EXTRACT FILES FROM TGZ

    if args.extract:
        for particle in args.particles:

            if release_details[args.release][particle]['file_type'] == 'wmp':
                extraction_dir = args.destination / 'wmp' / particle
            elif release_details[args.release][particle]['file_type'] == 'endf':
                extraction_dir = endf_files_dir / particle
            Path.mkdir(extraction_dir, parents=True, exist_ok=True)

            for f in release_details[args.release][particle]['compressed_files']:
                fname = Path(f).name
                # Extract files different depending on compression method
                if fname.endswith('.zip'):
                    print(f'Extracting {fname}...')
                    with zipfile.ZipFile(download_path / particle / fname) as zipf:
                        # Extracts files without folder structure in the zip file
                        for member in zipf.namelist():
                            filename = Path(member).name
                            # skip directories
                            if not filename:
                                continue
                            source = zipf.open(member)
                            target = open(extraction_dir / filename, "wb")
                            with source, target:
                                copyfileobj(source, target)
                elif fname.endswith('.tar.gz'):
                    with tarfile.open(download_path / particle / fname, 'r') as tgz:
                        print(f'Extracting {fname}...')
                        # extract files ignoring the internal folder structure
                        for member in tgz.getmembers():
                            if member.isreg():
                                member.name = Path(member.name).name
                                tgz.extract(member, path=extraction_dir)
                else:
                    # File is not compressed. Used for erratafiles. This ensures
                    # the n-005_B_010.endf erratafile overwrites the orginal
                    copy(download_path/particle/fname, extraction_dir/fname)

        if args.cleanup and download_path.exists():
            rmtree(download_path)

    # =========================================================================
    # PROCESS INCIDENT NEUTRON AND THERMAL SCATTERING DATA IN PARALLEL

    # Create output directory if it doesn't exist
    for particle in args.particles:
        particle_destination = args.destination / particle
        particle_destination.mkdir(parents=True, exist_ok=True)

    library = openmc.data.DataLibrary()

    if 'neutron' in args.particles:
        particle = 'neutron'
        with Pool() as pool:
            details = release_details[args.release][particle]
            results = []
            for filename in details['endf_files']:

                # Skip neutron evaluation that fails the processing stage
                if filename.name == 'n-000_n_001.endf':
                    continue

                func_args = (filename, args.destination / particle, args.libver,
                            args.temperatures)
                r = pool.apply_async(process_neutron, func_args)
                results.append(r)

            for path_neutron, path_thermal in details['sab_files']:
                func_args = (path_neutron, path_thermal,
                            args.destination / particle, args.libver)
                r = pool.apply_async(process_thermal, func_args)
                results.append(r)

            for r in results:
                r.wait()

        for p in sorted((args.destination / particle).glob('*.h5'), key=sort_key):
            library.register_file(p)


    # =========================================================================
    # INCIDENT PHOTON DATA

    if 'photon' in args.particles:
        particle = 'photon'
        details = release_details[args.release][particle]
        for photo_path, atom_path in zip(sorted(details['photo_files']),
                                        sorted(details['atom_files'])):
            # Generate instance of IncidentPhoton
            print('Converting:', photo_path.name, atom_path.name)
            data = openmc.data.IncidentPhoton.from_endf(photo_path, atom_path)

            # Export HDF5 file
            h5_file = args.destination / particle / f'{data.name}.h5'
            data.export_to_hdf5(h5_file, 'w', libver=args.libver)

            # Register with library
            library.register_file(h5_file)

    # =========================================================================
    # INCIDENT WMP NEUTRON DATA

    if 'wmp' in args.particles:
        for h5_file in Path(args.destination / 'wmp').rglob('*.h5'):
            library.register_file(h5_file)

    # Write cross_sections.xml
    library.export_to_xml(args.destination / 'cross_sections.xml')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""
Generate a depletion chain based on TENDL 2019 data. Note that TENDL 2019 does
not contain any decay or fission product yield (FPY) sublibraries, so these must
be borrowed from another library. The --lib flag for this script indicates what
library should be used for decay and FPY evaluations and defaults to JEFF 3.3.
"""

import json
import tarfile
from argparse import ArgumentParser
from pathlib import Path
from urllib.parse import urljoin
from zipfile import ZipFile

import openmc.data
import openmc.deplete as dep

from openmc_data.utils import download

# Parse command line arguments
parser = ArgumentParser()
parser.add_argument('--lib', choices=('jeff33', 'endf80'), default='jeff33',
                    help='Library to use for decay and fission product yields')
parser.add_argument('-r', '--release', choices=['2019', '2021'],
                    default='2021', help="The nuclear data library release "
                    "version. The currently supported options are 2019, "
                    "and 2021.")
parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default=None,
    help="filename of the chain file xml file produced. If left as None then "
    "the filename will follow this format 'chain_tendl_{release}_{lib}.xml'",
)
args = parser.parse_args()


def extract(filename, path=".", verbose=True):
    # Determine function to open archive
    if Path(filename).suffix == '.zip':
        func = ZipFile
    else:
        func = tarfile.open

    # Open archive and extract files
    with func(filename, 'r') as fh:
        if verbose:
            print(f'Extracting {filename} to {path}')
        fh.extractall(path)


def fix_jeff33_nfy(path):
    print(f'Fixing TPID in {path}...')
    new_path = path.with_name(path.name + '_fixed')
    if not new_path.exists():
        with path.open('r') as f:
            data = f.read()
        with new_path.open('w') as f:
            # Write missing TPID line
            f.write(" "*66 + "   1 0  0    0\n")
            f.write(data)
    return new_path


def main():

    library_name = 'tendl'

    cwd = Path.cwd()

    endf_files_dir = cwd.joinpath('-'.join([library_name, args.release, 'endf']))
    download_path = cwd.joinpath('-'.join([library_name, args.release, 'download']))

    neutron_dir = endf_files_dir / "neutrons"
    decay_dir = endf_files_dir / "decay"
    nfy_dir = endf_files_dir / "nfy"

    # This dictionary contains all the unique information about each release.
    # This can be extended to accommodated new releases
    release_details = {
        '2019': {
            'base_url': 'https://tendl.web.psi.ch/tendl_2019/tar_files/',
            'compressed_files': ['TENDL-n.tgz'],
            'transport_nuclides': 'depletion/tendl2019_nuclides.json',
            'neutron_files': endf_files_dir.glob('tendl19c/*'),
        },
        '2021': {
            'base_url': 'https://tendl.web.psi.ch/tendl_2021/tar_files/',
            'compressed_files': ['TENDL-n.tgz'],
            'transport_nuclides': 'depletion/tendl2021_nuclides.json',
            'neutron_files': endf_files_dir.glob('tendl21c/*'),
        }
    }

    DECAY_LIB = {
        'jeff33': 'https://www.oecd-nea.org/dbdata/jeff/jeff33/downloads/JEFF33-rdd.zip',
        'endf80': 'https://www.nndc.bnl.gov/endf-b8.0/zips/ENDF-B-VIII.0_decay.zip',
    }
    NFY_LIB = {
        'jeff33': 'https://www.oecd-nea.org/dbdata/jeff/jeff33/downloads/JEFF33-nfy.asc',
        'endf80': 'https://www.nndc.bnl.gov/endf-b8.0/zips/ENDF-B-VIII.0_nfy.zip',
    }


    # ==========================================================================
    # Incident neutron data
    for f in release_details[args.release]['compressed_files']:
        downloaded_file = download(
            url=urljoin(release_details[args.release]['base_url'], f),
            output_path=download_path
        )

    for f in release_details[args.release]['compressed_files']:
        extract(downloaded_file, neutron_dir)

    # Get list of transport nuclides in TENDL-2019
    with open(release_details[args.release]['transport_nuclides'], 'r') as fh:
        transport_nuclides = set(json.load(fh))

    neutron_files = [
        p
        for p in release_details[args.release]['neutron_files']
        if p.name[2:-6] in transport_nuclides  # filename is n-XXNNN.tendl
    ]

    # ==========================================================================
    # Decay and fission product yield data

    decay_zip = download(DECAY_LIB[args.lib], output_path=download_path)
    nfy_file = download(NFY_LIB[args.lib], output_path=download_path)

    extract(decay_zip, decay_dir)
    if args.lib == 'jeff33':
        decay_files = list(decay_dir.glob('*.ASC'))

        nfy_file_fixed = fix_jeff33_nfy(nfy_file)
        nfy_files = openmc.data.endf.get_evaluations(nfy_file_fixed)

    elif args.lib == 'endf80':
        decay_files = list(decay_dir.rglob('*.endf'))

        extract(nfy_file, nfy_dir)
        nfy_files = list(nfy_dir.rglob('*.endf'))

    chain = dep.Chain.from_endf(
        decay_files, nfy_files, neutron_files,
        reactions=dep.chain.REACTIONS.keys()
    )

    if args.destination is None:
        chain.export_to_xml(f'chain_{library_name}_{args.release}_{args.lib}.xml')
    else:
        chain.export_to_xml(args.destination)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import glob
import os
from argparse import ArgumentParser
from pathlib import Path
from zipfile import ZipFile
from collections import defaultdict
from io import StringIO

try:
    import lxml.etree as ET
    _have_lxml = True
except ImportError:
    import xml.etree.ElementTree as ET
    _have_lxml = False

import openmc.data
import openmc.deplete
from openmc.deplete.chain import REACTIONS, replace_missing_fpy
from openmc.deplete.nuclide import Nuclide, FissionYieldDistribution

from .casl_chain import CASL_CHAIN, UNMODIFIED_DECAY_BR
from openmc_data.utils import download

URLS = [
    'https://www.nndc.bnl.gov/endf-b7.1/zips/ENDF-B-VII.1-neutrons.zip',
    'https://www.nndc.bnl.gov/endf-b7.1/zips/ENDF-B-VII.1-decay.zip',
    'https://www.nndc.bnl.gov/endf-b7.1/zips/ENDF-B-VII.1-nfy.zip'
]

# Parse command line arguments
parser = ArgumentParser()
parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default='chain_casl.xml',
    help="filename of the chain file xml file produced.",
)
args = parser.parse_args()


def replace_missing_decay_product(product, decay_data, all_decay_data):
    # Determine atomic number, mass number, and metastable state
    Z, A, state = openmc.data.zam(product)
    symbol = openmc.data.ATOMIC_SYMBOL[Z]

    # Iterate until we find an existing nuclide in the chain
    while product not in decay_data:
        # If product has no decay data in the library, nothing further can be done
        if product not in all_decay_data:
            product = None
            break

        # If the current product is not in the chain but is stable, there's
        # nothing further we can do. Also, we only want to continue down the
        # decay chain if the half-life is short, so we also make a cutoff here
        # to terminate if the half-life is more than 1 day.
        decay_obj = all_decay_data[product]
        if decay_obj.nuclide['stable'] or decay_obj.half_life.n > 24*60*60:
            product = None
            break

        dominant_mode = max(decay_obj.modes, key=lambda x: x.branching_ratio)
        product = dominant_mode.daughter

    return product


def main():
    if os.path.isdir('./decay') and os.path.isdir('./nfy') and os.path.isdir('./neutrons'):
        endf_dir = '.'
    elif 'OPENMC_ENDF_DATA' in os.environ:
        endf_dir = os.environ['OPENMC_ENDF_DATA']
    else:
        for url in URLS:
            basename = download(url)
            with ZipFile(basename, 'r') as zf:
                print('Extracting {}...'.format(basename))
                zf.extractall()
        endf_dir = '.'

    decay_files = glob.glob(os.path.join(endf_dir, 'decay', '*.endf'))
    fpy_files = glob.glob(os.path.join(endf_dir, 'nfy', '*.endf'))
    neutron_files = glob.glob(os.path.join(endf_dir, 'neutrons', '*.endf'))

    # Create a Chain
    chain = openmc.deplete.Chain()

    print('Reading ENDF nuclear data from "{}"...'.format(os.path.abspath(endf_dir)))

    # Create dictionary mapping target to filename
    print('Processing neutron sub-library files...')
    reactions = {}
    for f in neutron_files:
        evaluation = openmc.data.endf.Evaluation(f)
        nuc_name = evaluation.gnd_name
        if nuc_name in CASL_CHAIN:
            reactions[nuc_name] = {}
            for mf, mt, nc, mod in evaluation.reaction_list:
                # Q value for each reaction is given in MF=3
                if mf == 3:
                    file_obj = StringIO(evaluation.section[3, mt])
                    openmc.data.endf.get_head_record(file_obj)
                    q_value = openmc.data.endf.get_cont_record(file_obj)[1]
                    reactions[nuc_name][mt] = q_value

    # Determine what decay and FPY nuclides are available
    print('Processing decay sub-library files...')
    decay_data = {}
    all_decay_data = {}
    for f in decay_files:
        decay_obj = openmc.data.Decay(f)
        nuc_name = decay_obj.nuclide['name']
        all_decay_data[nuc_name] = decay_obj
        if nuc_name in CASL_CHAIN:
            decay_data[nuc_name] = decay_obj

    for nuc_name in CASL_CHAIN:
        if nuc_name not in decay_data:
            print('WARNING: {} has no decay data!'.format(nuc_name))

    print('Processing fission product yield sub-library files...')
    fpy_data = {}
    for f in fpy_files:
        fpy_obj = openmc.data.FissionProductYields(f)
        name = fpy_obj.nuclide['name']
        if name in CASL_CHAIN:
            fpy_data[name] = fpy_obj

    print('Creating depletion_chain...')
    missing_daughter = []
    missing_rx_product = []
    missing_fpy = []

    for idx, parent in enumerate(sorted(decay_data, key=openmc.data.zam)):
        data = decay_data[parent]

        nuclide = Nuclide(parent)

        chain.nuclides.append(nuclide)
        chain.nuclide_dict[parent] = idx

        if not CASL_CHAIN[parent][0] and \
           not data.nuclide['stable'] and data.half_life.nominal_value != 0.0:
            nuclide.half_life = data.half_life.nominal_value
            nuclide.decay_energy = data.decay_energy.nominal_value
            sum_br = 0.0
            for mode in data.modes:
                decay_type = ','.join(mode.modes)
                if mode.daughter in decay_data:
                    target = mode.daughter
                else:
                    missing_daughter.append((parent, mode))
                    continue

                # Append decay mode
                br = mode.branching_ratio.nominal_value
                nuclide.add_decay_mode(decay_type, target, br)

            # Ensure sum of branching ratios is unity by slightly modifying last
            # value if necessary
            sum_br = sum(m.branching_ratio for m in nuclide.decay_modes)
            if sum_br != 1.0 and nuclide.decay_modes and parent not in UNMODIFIED_DECAY_BR:
                decay_type, target, br = nuclide.decay_modes.pop()
                br = 1.0 - sum(m.branching_ratio for m in nuclide.decay_modes)
                nuclide.add_decay_mode(decay_type, target, br)

        # If nuclide has incident neutron data, we need to list what
        # transmutation reactions are possible
        fissionable = False
        transmutation_reactions = ('(n,2n)', '(n,3n)', '(n,4n)', '(n,gamma)',
                                   '(n,p)', '(n,a)')
        if parent in reactions:
            reactions_available = reactions[parent].keys()
            for name in transmutation_reactions:
                mts, changes, _ = REACTIONS[name]
                if mts & reactions_available:
                    delta_A, delta_Z = changes
                    A = data.nuclide['mass_number'] + delta_A
                    Z = data.nuclide['atomic_number'] + delta_Z
                    daughter = '{}{}'.format(openmc.data.ATOMIC_SYMBOL[Z], A)

                    if daughter not in decay_data:
                        daughter = replace_missing_decay_product(
                            daughter, decay_data, all_decay_data)
                        if daughter is None:
                            missing_rx_product.append((parent, name, daughter))

                    # Store Q value -- use sorted order so we get summation
                    # reactions (e.g., MT=103) first
                    for mt in sorted(mts):
                        if mt in reactions[parent]:
                            q_value = reactions[parent][mt]
                            break
                    else:
                        q_value = 0.0

                    nuclide.add_reaction(name, daughter, q_value, 1.0)

            # Check for fission reactions
            if any(mt in reactions_available for mt in [18, 19, 20, 21, 38]):
                q_value = reactions[parent][18]
                nuclide.add_reaction('fission', None, q_value, 1.0)
                fissionable = True

        if fissionable:
            if parent in fpy_data:
                fpy = fpy_data[parent]
            else:
                nuclide._fpy = replace_missing_fpy(parent, fpy_data, decay_data)
                missing_fpy.append((parent, nuclide._fpy))
                continue

            if fpy.energies is not None:
                yield_energies = fpy.energies
            else:
                yield_energies = [0.0]

            yield_data = {}
            for E, table_yd, table_yc in zip(yield_energies, fpy.independent, fpy.cumulative):
                yields = defaultdict(float)
                for product in table_yd:
                    if product in decay_data:
                        # identifier
                        ifpy = CASL_CHAIN[product][2]
                        # 1 for independent
                        if ifpy == 1:
                            if product not in table_yd:
                                print('No independent fission yields found for {} in {}'.format(product, parent))
                            else:
                                yields[product] += table_yd[product].nominal_value
                        # 2 for cumulative
                        elif ifpy == 2:
                            if product not in table_yc:
                                print('No cumulative fission yields found for {} in {}'.format(product, parent))
                            else:
                                yields[product] += table_yc[product].nominal_value
                        # 3 for special treatment with weight fractions
                        elif ifpy == 3:
                            for name_i, weight_i, ifpy_i in CASL_CHAIN[product][3]:
                                if name_i not in table_yd:
                                    print('No fission yields found for {} in {}'.format(name_i, parent))
                                else:
                                    if ifpy_i == 1:
                                        yields[product] += weight_i * table_yd[name_i].nominal_value
                                    elif ifpy_i == 2:
                                        yields[product] += weight_i * table_yc[name_i].nominal_value

                yield_data[E] = yields

            nuclide.yield_data = FissionYieldDistribution(yield_data)

    # Replace missing FPY data
    for nuclide in chain.nuclides:
        if hasattr(nuclide, '_fpy'):
            nuclide.yield_data = chain[nuclide._fpy].yield_data

    # Display warnings
    if missing_daughter:
        print('The following decay modes have daughters with no decay data:')
        for parent, mode in missing_daughter:
            print('  {} -> {} ({})'.format(parent, mode.daughter, ','.join(mode.modes)))
        print('')

    if missing_rx_product:
        print('The following reaction products have no decay data:')
        for vals in missing_rx_product:
            print('{} {} -> {}'.format(*vals))
        print('')

    if missing_fpy:
        print('The following fissionable nuclides have no fission product yields:')
        for parent, replacement in missing_fpy:
            print('  {}, replaced with {}'.format(parent, replacement))
        print('')

    chain.export_to_xml(args.destination)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""
Downloads preprocessed ENDF HDF5 files and cross_sections.xml from openmc.org
for use with OpenMC.
"""

import argparse
from pathlib import Path
from urllib.parse import urljoin

from openmc_data.urls_h5 import all_h5_release_details
from openmc_data.utils import download, extract, state_download_size


class CustomFormatter(
    argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter
):
    pass


parser = argparse.ArgumentParser(description=__doc__, formatter_class=CustomFormatter)
parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default=None,
    help="Directory to create new library in",
)
parser.add_argument("--download", action="store_true", help="Download files from PSI")
parser.add_argument(
    "--no-download",
    dest="download",
    action="store_false",
    help="Do not download files from PSI",
)
parser.add_argument("--extract", action="store_true", help="Extract tar/zip files")
parser.add_argument(
    "--no-extract",
    dest="extract",
    action="store_false",
    help="Do not extract tar/zip files",
)
parser.add_argument(
    "-r",
    "--release",
    choices=["b7.1"],
    default="b7.1",
    help="The nuclear data library release version. The currently supported "
         "options are b7.1",
)
parser.add_argument(
    "--cleanup",
    action="store_true",
    help="Remove download directories when data has " "been processed",
)
parser.add_argument(
    "--no-cleanup",
    dest="cleanup",
    action="store_false",
    help="Do not remove download directories when data has " "been processed",
)
parser.set_defaults(download=True, extract=True, cleanup=False)
args = parser.parse_args()


def main():

    library_name = 'nndc'
    details = all_h5_release_details[library_name][args.release]["neutron-photon"]

    cwd = Path.cwd()

    download_path = cwd.joinpath("-".join([library_name, args.release, "download"]))

    if args.destination is None:
        args.destination = Path("-".join([library_name, args.release, "hdf5"]))

    if args.download:
        state_download_size(
            details['compressed_file_size'],
            details['uncompressed_file_size'],
            'GB'
        )
        download(
            urljoin(details["base_url"], details["compressed_files"][0]),
            output_path=download_path,
        )

    if args.extract:
        extract(
            compressed_files=[
                download_path / f for f in details["compressed_files"]
            ],
            extraction_dir=args.destination,
            del_compressed_file=args.cleanup,
        )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
Generate JEFF 3.3 HDF5 library for use in OpenMC by first processing ENDF files
using NJOY. The resulting library will contain incident neutron, photoatomic,
and thermal scattering data. Note that JEFF doesn't distribute photoatomic or
atomic relaxation data so these are obtained from ENDF/B-VIII.0.

"""

import argparse
import os
import sys
import tarfile
import tempfile
import zipfile
from multiprocessing import Pool
from pathlib import Path
from urllib.parse import urljoin

import openmc.data

from openmc_data import download, process_neutron, process_thermal

# Make sure Python version is sufficient
assert sys.version_info >= (3, 6), "Python 3.6+ is required"


class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter,
                      argparse.RawDescriptionHelpFormatter):
    pass


parser = argparse.ArgumentParser(
    description=__doc__,
    formatter_class=CustomFormatter
)
parser.add_argument('-d', '--destination', type=Path, default=Path('jeff33_hdf5'),
                    help='Directory to create new library in')
parser.add_argument('--libver', choices=['earliest', 'latest'],
                    default='earliest', help="Output HDF5 versioning. Use "
                    "'earliest' for backwards compatibility or 'latest' for "
                    "performance")
parser.add_argument('--download', action='store_true',
                    help='Download zip files from NNDC')
parser.add_argument('--no-download', dest='download', action='store_false',
                    help='Do not download zip files from NNDC')
parser.add_argument('--use-tmpdir', dest='tmpdir', action='store_true',
                    help='Use temporary directory while processing')
parser.add_argument('--no-use-tmpdir', dest='tmpdir', action='store_false',
                    help='Do not use temporary directory while processing')
parser.add_argument('--extract', action='store_true',
                    help='Extract zip files')
parser.add_argument('--no-extract', dest='extract', action='store_false',
                    help='Do not extract zip files')
parser.add_argument('--temperatures', type=float,
                    default=[250.0, 293.6, 600.0, 900.0, 1200.0, 2500.0],
                    help="Temperatures in Kelvin", nargs='+')
parser.set_defaults(download=True, extract=True, tmpdir=True)
args = parser.parse_args()


def sort_key(path):
    if path.name.startswith('c_'):
        # Ensure that thermal scattering gets sorted after neutron data
        return (1000, path)
    else:
        return openmc.data.zam(path.stem)


def main():

    base_endf = 'https://www.nndc.bnl.gov/endf-b8.0/zips/'
    base_jeff = 'http://www.oecd-nea.org/dbdata/jeff/jeff33/downloads/'
    files = [
        (base_jeff, 'JEFF33-n.tgz', 'e540bbf95179257280c61acfa75c83de'),
        (base_jeff, 'JEFF33-tsl.tgz', '82a6df4cb802aa4a09b95309f7861c54'),
        (base_endf, 'ENDF-B-VIII.0_photoat.zip', 'd49f5b54be278862e1ce742ccd94f5c0'),
        (base_endf, 'ENDF-B-VIII.0_atomic_relax.zip', 'e04d50098cb2a7e4fe404ec4071611cc'),
    ]

    tendl_files = [
        'https://tendl.web.psi.ch/tendl_2019/neutron_file/C/C013/lib/endf/n-C013.tendl',
        'https://tendl.web.psi.ch/tendl_2019/neutron_file/O/O017/lib/endf/n-O017.tendl',
    ]

    neutron_dir = Path('endf6')
    thermal_dir = Path('JEFF33-tsl')
    photoat_dir = Path('ENDF-B-VIII.0_photoat')
    atomic_dir = Path('ENDF-B-VIII.0_atomic_relax')
    thermal_paths = [
        (neutron_dir / '1-H-1g.jeff33', thermal_dir / 'tsl-HinCaH2.jeff33'),
        (neutron_dir / '1-H-1g.jeff33', thermal_dir / 'tsl-HinCH2.jeff33'),
        (neutron_dir / '1-H-1g.jeff33', thermal_dir / 'tsl-HinH2O.jeff33'),
        (neutron_dir / '1-H-1g.jeff33', thermal_dir / 'tsl-HinIce.jeff33'),
        (neutron_dir / '1-H-1g.jeff33', thermal_dir / 'tsl-HinMesitylene-PhaseII.jeff33'),
        (neutron_dir / '1-H-1g.jeff33', thermal_dir / 'tsl-HinOrthoH.jeff33'),
        (neutron_dir / '1-H-1g.jeff33', thermal_dir / 'tsl-HinParaH.jeff33'),
        (neutron_dir / '1-H-1g.jeff33', thermal_dir / 'tsl-HinToluene.jeff33'),
        (neutron_dir / '1-H-1g.jeff33', thermal_dir / 'tsl-HinZrH.jeff33'),
        (neutron_dir / '1-H-2g.jeff33', thermal_dir / 'tsl-DinD2O.jeff33'),
        (neutron_dir / '1-H-2g.jeff33', thermal_dir / 'tsl-DinOrthoD.jeff33'),
        (neutron_dir / '1-H-2g.jeff33', thermal_dir / 'tsl-DinParaD.jeff33'),
        (neutron_dir / '4-Be-9g.jeff33', thermal_dir / 'tsl-Be.jeff33'),
        (neutron_dir / '6-C-0g.jeff33', thermal_dir / 'tsl-Graphite.jeff33'),
        (neutron_dir / '8-O-16g.jeff33', thermal_dir / 'tsl-O16inAl2O3.jeff33'),
        (neutron_dir / '8-O-16g.jeff33', thermal_dir / 'tsl-OinD2O.jeff33'),
        (neutron_dir / '12-Mg-24g.jeff33', thermal_dir / 'tsl-Mg.jeff33'),
        (neutron_dir / '13-Al-27g.jeff33', thermal_dir / 'tsl-Al27inAl2O3.jeff33'),
        (neutron_dir / '14-Si-28g.jeff33', thermal_dir / 'tsl-Silicon.jeff33'),
        (neutron_dir / '20-Ca-40g.jeff33', thermal_dir / 'tsl-CainCaH2.jeff33'),
    ]

    pwd = Path.cwd()

    destination = args.destination.resolve()
    (destination / 'photon').mkdir(parents=True, exist_ok=True)

    with tempfile.TemporaryDirectory() as tmpdir:
        # Save current working directory and temporarily change dir
        if args.tmpdir:
            os.chdir(tmpdir)
        library = openmc.data.DataLibrary()

        # =========================================================================
        # DOWNLOAD FILES
        if args.download:
            for base, fname, checksum in files:
                download(urljoin(base, fname), checksum)

        # =========================================================================
        # EXTRACT ARCHIVES

        if args.extract:
            for _, f, _ in files:
                if f.endswith('.zip'):
                    print(f'Extracting {f}...')
                    with zipfile.ZipFile(f) as zipf:
                        zipf.extractall()
                elif f.endswith('.tar.gz') or f.endswith('.tgz'):
                    print(f'Extracting {f}...')
                    with tarfile.open(f) as tgz:
                        tgz.extractall()

        # =========================================================================
        # REPLACE C13 AND O17 WITH FILES FROM TENDL 2019

        # The evaluations for C13 and O17 can't be processed by recent versions of
        # NJOY 2016 because of a bug in MF=6 in the evaluations. Instead of trying
        # to patch the evaluation, we just replace it with more recent evaluations
        # from TENDL 2019

        for url in tendl_files:
            download(url, output_path=neutron_dir)

        (neutron_dir / 'n-C013.tendl').rename(neutron_dir / '6-C-13g.jeff33')
        (neutron_dir / 'n-O017.tendl').rename(neutron_dir / '8-O-17g.jeff33')

        # =========================================================================
        # PROCESS INCIDENT NEUTRON AND THERMAL SCATTERING DATA IN PARALLEL

        with Pool() as pool:
            neutron_paths = neutron_dir.glob('*.jeff33')
            results = []
            for p in neutron_paths:
                func_args = (p, destination, args.libver, args.temperatures)
                r = pool.apply_async(process_neutron, func_args)
                results.append(r)
            for p_neut, p_therm in thermal_paths:
                func_args = (p_neut, p_therm, destination, args.libver)
                r = pool.apply_async(process_thermal, func_args)
                results.append(r)
            for r in results:
                r.wait()

        for p in sorted(destination.glob('*.h5'), key=sort_key):
            library.register_file(p)

        # =========================================================================
        # INCIDENT PHOTON DATA

        for z in range(1, 101):
            element = openmc.data.ATOMIC_SYMBOL[z]
            print('Generating HDF5 file for Z={} ({})...'.format(z, element))

            # Generate instance of IncidentPhoton
            photo_file = photoat_dir / f'photoat-{z:03}_{element}_000.endf'
            atom_file = atomic_dir / f'atom-{z:03}_{element}_000.endf'
            data = openmc.data.IncidentPhoton.from_endf(photo_file, atom_file)

            # Write HDF5 file and register it
            outfile = destination / 'photon' / f'{element}.h5'
            data.export_to_hdf5(outfile, 'w', libver=args.libver)
            library.register_file(outfile)

        library.export_to_xml(destination / 'cross_sections.xml')

        # Change back to original directory
        if args.tmpdir:
            os.chdir(pwd)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import argparse
import os
import sys
from multiprocessing import Pool
import re
import shutil
from pathlib import Path

import openmc.data


description = """
This scripts generates random (gaussian) evaluations of a nuclear data file following 
its covariance matrix using SANDY, and converts them to HDF5 for use in OpenMC. Script
generates a cross_sections_sandy.xml file with the standard library plus the sampled evaluations.
"""


class CustomFormatter(
    argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter
):
    pass


parser = argparse.ArgumentParser(
    description=description, formatter_class=CustomFormatter)

parser.add_argument("-n", "--nuclides", nargs="+", 
                    default=["Fe56"], help="The nuclide(s) to be sampled")
parser.add_argument("-d", "--destination", default=None, 
                    help="Directory to create new library in")
parser.add_argument("-l", "--libdir", default=None, 
                    help="Directory of endf library to sample eg. nndc-b7.1-endf folder")
parser.add_argument("-x", "--xlib", default=None, 
                    help="cross_section.xml library to add random evaluations to. Default is OPENMC_CROSS_SECTIONS")
parser.add_argument("-s", "--samples", default=200, 
                    help="Number of samples per nuclide")
parser.add_argument("-p", "--processes", default=1, 
                    help="number of worker processes (default = 1)")
parser.add_argument("-f", "--format_only", default=False,
                    help="Only format previously sampled files to HDF5")

args = parser.parse_args()


def main():

    script_dir = Path.cwd()

    output_dir = args.destination
    if output_dir == None:
        output_dir = script_dir / "sandy_rand"
    else:
        output_dir = Path(output_dir).resolve()

    endf_files_dir = output_dir / "endf"
    hdf5_files_dir = output_dir / "hdf5"

    libdir = args.libdir
    if libdir == None:
        raise Exception("Directory of ENDF library required for sampling, eg. nndc-b7.1-endf folder. Use -l prefix to specify")
    else:
        libdir = Path(libdir).resolve()

    xlib = args.xlib
    if xlib == None:
        xlib = os.getenv("OPENMC_CROSS_SECTIONS")
    else:
        xlib = Path(xlib).resolve()

    nuclides = args.nuclides

    format_only = args.format_only

    # ==============================================================================
    # CHECK IF REQUEST IS VALID AND IF ENDF FILES EXIST

    prefix = "n-"
    suffix = ".endf"

    atomic_dict = openmc.data.ATOMIC_NUMBER
    nuc_dict = {}

    for nuc in nuclides:
        mass_num = int(re.findall("(\d+)", nuc)[0])
        atomic_sym = "".join([i for i in nuc if not i.isdigit()])
        if atomic_sym not in atomic_dict.keys():
            print(f"Entered nuclide {nuc} does not have a valid atomic symbol")
            sys.exit()
        atomic_num = atomic_dict[atomic_sym]

        file_mass = f"{mass_num:03}"
        file_atomic = f"{atomic_num:03}"

        file_name = f"{prefix}{file_atomic}_{atomic_sym}_{file_mass}{suffix}"

        if not (libdir / "neutron" / file_name).is_file():
            print(f"File {libdir / 'neutron' / file_name} does not exist")
            sys.exit()
        nuc_dict[nuc] = {
            "sym": atomic_sym,
            "mass_num": mass_num,
            "atomic_num": atomic_num,
            "file_name": file_name,
        }

    # ==============================================================================
    # GENERATE RANDOM EVALUATIONS OF NUCLEAR DATA USING SANDY

    if not format_only:
        output_dir.mkdir(exist_ok=True)
        endf_files_dir.mkdir(exist_ok=True)

        for nuc in nuclides:

            nuc_dir_endf = endf_files_dir / nuc
            nuc_dir_endf.mkdir(exist_ok=True)

            shutil.copyfile(
                libdir / "neutron" / nuc_dict[nuc]["file_name"],
                nuc_dir_endf / nuc_dict[nuc]["file_name"],
            )
            os.chdir(nuc_dir_endf)
            sandy_command = f"sandy {nuc_dict[nuc]['file_name']} --samples {args.samples} --outname {nuc} --processes {args.processes}"
            os.system(sandy_command)

        os.chdir(script_dir)

    # ==============================================================================
    # CONVERT RANDOM EVALUATIONS TO HDF5


    def process_neutron_random(nuc, i, out_dir, in_dir, file_num):  # Need to add temperatures
        """Process ENDF neutron sublibrary file into HDF5 and write into a
        specified output directory."""

        fileIn = in_dir / f"{nuc}-{i}"
        fileOut = out_dir / f"{nuc}-{i}.h5"

        data = openmc.data.IncidentNeutron.from_njoy(fileIn)
        data.name = f"{nuc}-{i}"
        data.export_to_hdf5(fileOut, "w")
        if i % 40 == 0:
            print(f"Nuclide {nuc} {i+1}/{file_num} finished")


    print("Beginning NJOY processing")
    with Pool() as pool:
        results = []
        file_num = int(args.samples)
        for nuc in nuclides:

            in_dir = endf_files_dir / nuc
            out_dir = hdf5_files_dir / nuc

            hdf5_files_dir.mkdir(exist_ok=True)
            out_dir.mkdir(exist_ok=True)

            print(f"Beginning nuclide {nuc} ...")
            for i in range(1, file_num + 1):
                func_args = (nuc, i, out_dir, in_dir, file_num)
                r = pool.apply_async(process_neutron_random, func_args)
                results.append(r)

        for r in results:
            r.wait()


    # ==============================================================================
    # Create xml library

    lib = openmc.data.DataLibrary()
    lib = lib.from_xml(xlib)  # Gets current

    for nuc in nuclides:
        out_dir = hdf5_files_dir / nuc
        for i in range(1, file_num + 1):
            fileOut = out_dir / f"{nuc}-{i}.h5"
            lib.register_file(fileOut)

    pre = output_dir / "cross_sections_pre.xml"
    post = output_dir / "cross_sections_sandy.xml"

    lib.export_to_xml(pre)
    if post.exists():
        command = f"combine_libraries.py -l {pre} {post} -o {post}"
        os.system(command)
    else:
        lib.export_to_xml(post)

    pre.unlink()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""
Download CENDL 3.1 data from OECD NEA or CENDL-3.2 from CNDC
and convert it to a HDF5 library for use with OpenMC.
"""

import argparse
from multiprocessing import Pool
from pathlib import Path
from urllib.parse import urljoin

import openmc.data
from openmc_data import download, extract, process_neutron, state_download_size


class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter,
                      argparse.RawDescriptionHelpFormatter):
    pass


parser = argparse.ArgumentParser(
    description=__doc__,
    formatter_class=CustomFormatter
)
parser.add_argument('-d', '--destination', type=Path, default=None,
                    help='Directory to create new library in')
parser.add_argument('--download', action='store_true',
                    help='Download files from OECD-NEA')
parser.add_argument('--no-download', dest='download', action='store_false',
                    help='Do not download files from OECD-NEA')
parser.add_argument('--extract', action='store_true',
                    help='Extract tar/zip files')
parser.add_argument('--no-extract', dest='extract', action='store_false',
                    help='Do not extract tar/zip files')
parser.add_argument('--libver', choices=['earliest', 'latest'],
                    default='latest', help="Output HDF5 versioning. Use "
                    "'earliest' for backwards compatibility or 'latest' for "
                    "performance")
parser.add_argument('-r', '--release', choices=['3.1', '3.2'],
                    default='3.2', help="The nuclear data library release "
                    "version. The currently supported options are 3.1, 3.2")
parser.add_argument('--cleanup', action='store_true',
                    help="Remove download directories when data has "
                    "been processed")
parser.add_argument('--no-cleanup', dest='cleanup', action='store_false',
                    help="Do not remove download directories when data has "
                    "been processed")
parser.set_defaults(download=True, extract=True, cleanup=False)
args = parser.parse_args()


def main():

    library_name = 'cendl'

    cwd = Path.cwd()

    endf_files_dir = cwd.joinpath('-'.join([library_name, args.release, 'endf']))
    download_path = cwd.joinpath('-'.join([library_name, args.release, 'download']))
    # the destination is decided after the release is known
    # to avoid putting the release in a folder with a misleading name
    if args.destination is None:
        args.destination = Path('-'.join([library_name, args.release, 'hdf5']))

    # This dictionary contains all the unique information about each release.
    # This can be extended to accommodate new releases
    release_details = {
        '3.1': {
            'base_url': 'https://www.oecd-nea.org/dbforms/data/eva/evatapes/cendl_31/',
            'compressed_files': ['CENDL-31.zip'],
            'neutron_files': endf_files_dir.glob('*.C31'),
            'metastables': endf_files_dir.glob('*m.C31'),
            'compressed_file_size': '0.03',
            'uncompressed_file_size': '0.4'
        },
        '3.2': {
            'base_url': 'http://www.nuclear.csdb.cn/endf/CENDL/',
            'compressed_files': ['n-CENDL-3.2.zip'],
            'neutron_files': endf_files_dir.glob('n-CENDL-3.2/CENDL-3.2/*.C32'),
            'metastables': endf_files_dir.glob('n-CENDL-3.2/CENDL-3.2/*m.C32'),
            'compressed_file_size': '0.11',
            'uncompressed_file_size': '0.41'
        }
    }


    # ==============================================================================
    # DOWNLOAD FILES FROM WEBSITE

    if args.download:
        state_download_size(
            release_details[args.release]['compressed_file_size'],
            release_details[args.release]['uncompressed_file_size'],
            'GB'
        )
        for f in release_details[args.release]['compressed_files']:
            # Establish connection to URL
            download(urljoin(release_details[args.release]['base_url'], f),
                    output_path=download_path)


    # ==============================================================================
    # EXTRACT FILES FROM ZIP
    if args.extract:
        extract(
                compressed_files=[download_path/ f for f in release_details[args.release]['compressed_files']],
                extraction_dir=endf_files_dir,
                del_compressed_file=args.cleanup
            )


    # ==============================================================================
    # GENERATE HDF5 LIBRARY -- NEUTRON FILES

    # Get a list of all ENDF files
    neutron_files = release_details[args.release]['neutron_files']

    # Create output directory if it doesn't exist
    args.destination.mkdir(parents=True, exist_ok=True)

    library = openmc.data.DataLibrary()

    with Pool() as pool:
        results = []
        for filename in sorted(neutron_files):

            # this is a fix for the CENDL 3.1 release where the
            # 22-Ti-047.C31 and 5-B-010.C31 files contain non-ASCII characters
            if library_name == 'cendl' and args.release == '3.1' and filename.name in ['22-Ti-047.C31', '5-B-010.C31']:
                print('Manual fix for incorrect value in ENDF file')
                text = open(filename, 'rb').read().decode('utf-8', 'ignore').split('\r\n')
                if filename.name == '22-Ti-047.C31':
                    text[205] = ' 8) YUAN Junqian,WANG Yongchang,etc.               ,16,(1),57,92012228 1451  205'
                if filename.name == '5-B-010.C31':
                    text[203] = '21)   Day R.B. and Walt M.  Phys.rev.117,1330 (1960)               525 1451  203'
                open(filename, 'w').write('\r\n'.join(text))

            func_args = (filename, args.destination, args.libver)
            r = pool.apply_async(process_neutron, func_args)
            results.append(r)

        for r in results:
            r.wait()

    # Register with library
    for p in sorted((args.destination).glob('*.h5')):
        library.register_file(p)

    # Write cross_sections.xml
    library.export_to_xml(args.destination / 'cross_sections.xml')


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from pathlib import Path
import tarfile

import numpy as np
import h5py

from openmc_data.utils import download

def main():

    base_url = 'http://geant4.cern.ch/support/source/'
    version = '6.48'
    filename = f'G4EMLOW.{version}.tar.gz'

    # ==============================================================================
    # DOWNLOAD FILES FROM GEANT4 SITE

    download(base_url + filename)

    # ==============================================================================
    # EXTRACT FILES FROM TGZ

    g4dir = Path(f'G4EMLOW{version}')
    if not g4dir.is_dir():
        with tarfile.open(filename, 'r') as tgz:
            print(f'Extracting {filename}...')
            tgz.extractall()

    # ==============================================================================
    # GENERATE COMPTON PROFILE HDF5 FILE

    print('Generating compton_profiles.h5...')

    shell_file = g4dir / 'doppler' / 'shell-doppler.dat'

    with open(shell_file, 'r') as shell, h5py.File('compton_profiles.h5', 'w') as f:
        # Read/write electron momentum values
        pz = np.loadtxt(g4dir / 'doppler' / 'p-biggs.dat')
        f.create_dataset('pz', data=pz)

        for z in range(1, 101):
            # Create group for this element
            group = f.create_group(f'{z:03}')

            # Read data into one long array
            path = g4dir / 'doppler' / f'profile-{z}.dat'
            with open(path, 'r') as profile:
                j = np.fromstring(profile.read(), sep=' ')

            # Determine number of electron shells and reshape. Profiles are
            # tabulated against a grid of 31 momentum values.
            n_shells = j.size // 31
            j.shape = (n_shells, 31)

            # Write Compton profile for this Z
            group.create_dataset('J', data=j)

            # Determine binding energies and number of electrons for each shell
            num_electrons = []
            binding_energy = []
            while True:
                words = shell.readline().split()
                if words[0] == '-1':
                    break
                num_electrons.append(float(words[0]))
                binding_energy.append(float(words[1]))

            # Write binding energies and number of electrons
            group.create_dataset('num_electrons', data=num_electrons)
            group.create_dataset('binding_energy', data=binding_energy)


if __name__ == '__main__':
    main()



all_h5_release_details = {
    "nndc": {
        "b7.1": {
            "neutron-photon": {
                "base_url": "https://anl.box.com/shared/static/",
                "compressed_files": ["9igk353zpy8fn9ttvtrqgzvw1vtejoz6.xz"],
                "compressed_file_size": 1.6,
                "uncompressed_file_size": 6.2,
            }
        },
    }
}



#!/usr/bin/env python

"""
Download ENDF/B-VII.1 incident neutron ACE data and incident photon ENDF data
from NNDC and convert it to an HDF5 library for use with OpenMC. This data is
used for OpenMC's regression test suite.
"""

import argparse
import sys
from pathlib import Path

import openmc.data
from openmc_data import download, extract, state_download_size, all_release_details

# Make sure Python version is sufficient
assert sys.version_info >= (3, 6), "Python 3.6+ is required"


class CustomFormatter(
    argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter
):
    pass


parser = argparse.ArgumentParser(description=__doc__, formatter_class=CustomFormatter)

parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default="nndc-b7.1-hdf5",
    help="Directory to create new library in",
)
parser.add_argument(
    "--download", action="store_true", help="Download tarball from NNDC-BNL"
)
parser.add_argument(
    "--no-download",
    dest="download",
    action="store_false",
    help="Do not download tarball from NNDC-BNL",
)
parser.add_argument("--extract", action="store_true", help="Extract compressed files")
parser.add_argument(
    "--no-extract",
    dest="extract",
    action="store_false",
    help="Do not extract compressed file if it has already been extracted",
)
parser.add_argument(
    "--libver",
    choices=["earliest", "latest"],
    default="earliest",
    help="Output HDF5 versioning. Use "
    "'earliest' for backwards compatibility or 'latest' for "
    "performance",
)
parser.add_argument(
    "-p",
    "--particles",
    choices=["neutron", "photon"],
    nargs="+",
    default=["neutron", "photon"],
    help="Incident particles to include",
)
parser.add_argument(
    "--cleanup",
    action="store_true",
    help="Remove download directories when data has " "been processed",
)
parser.add_argument(
    "--no-cleanup",
    dest="cleanup",
    action="store_false",
    help="Do not remove download directories when data has " "been processed",
)
parser.set_defaults(download=True, extract=True, cleanup=False)
args = parser.parse_args()


def main():

    library_name = "nndc"
    release = "b7.1"

    cwd = Path.cwd()

    ace_files_dir = Path("-".join([library_name, release, "ace"]))
    endf_files_dir = Path("-".join([library_name, release, "endf"]))
    download_path = cwd.joinpath("-".join([library_name, release, "download"]))

    # This dictionary contains all the unique information about each release. This
    # can be exstened to accommodated new releases
    release_details = all_release_details[library_name][release]

    compressed_file_size, uncompressed_file_size = 0, 0
    for p in ("neutron", "photon"):
        compressed_file_size += release_details[p]["compressed_file_size"]
        uncompressed_file_size += release_details[p]["uncompressed_file_size"]

    # ==============================================================================
    # DOWNLOAD FILES FROM NNDC SITE

    if args.download:
        state_download_size(
            compressed_file_size,
            uncompressed_file_size,
            'MB'
        )
        for particle in args.particles:
            particle_download_path = download_path / particle
            for f, checksum in zip(
                release_details[particle]["compressed_files"],
                release_details[particle]["checksums"],
            ):
                # Establish connection to URL
                url = release_details[particle]["base_url"] + f
                downloaded_file = download(
                    url, output_path=particle_download_path, checksum=checksum
                )

    # ==============================================================================
    # EXTRACT FILES FROM TGZ

    if args.extract:
        for particle in args.particles:
            if release_details[particle]["file_type"] == "ace":
                extraction_dir = ace_files_dir
            elif release_details[particle]["file_type"] == "endf":
                extraction_dir = endf_files_dir

            extract(
                compressed_files=[
                    download_path / particle / f
                    for f in release_details[particle]["compressed_files"]
                ],
                extraction_dir=extraction_dir,
                del_compressed_file=args.cleanup,
            )

    # ==============================================================================
    # FIX ZAID ASSIGNMENTS FOR VARIOUS S(A,B) TABLES

    if "neutron" in args.particles:
        print("Fixing ZAIDs for S(a,b) tables")
        fixes = [("bebeo.acer", "8016", "   0"), ("obeo.acer", "4009", "   0")]
        for table, old, new in fixes:
            filename = ace_files_dir / table
            with open(filename, "r") as fh:
                text = fh.read()
            text = text.replace(old, new, 1)
            with open(filename, "w") as fh:
                fh.write(text)

    # ==============================================================================
    # GENERATE HDF5 LIBRARY

    # Create output directory if it doesn't exist
    for particle in args.particles:
        particle_destination = args.destination / particle
        particle_destination.mkdir(parents=True, exist_ok=True)

    library = openmc.data.DataLibrary()

    for particle in args.particles:
        details = release_details[particle]
        if particle == "neutron":
            for cls, files in [
                (openmc.data.IncidentNeutron, ace_files_dir.rglob(details["ace_files"])),
                (openmc.data.ThermalScattering, ace_files_dir.rglob(details["sab_files"])),
            ]:
                for path in sorted(files):
                    print(f"Converting: {path.name}")
                    data = cls.from_ace(path)
                    # Export HDF5 file
                    h5_file = args.destination.joinpath(particle, data.name + ".h5")
                    data.export_to_hdf5(h5_file, "w", libver=args.libver)

                    # Register with library
                    library.register_file(h5_file)

        elif particle == "photon":
            for photo_path, atom_path in zip(
                sorted(endf_files_dir.glob(details["photo_files"])), sorted(endf_files_dir.glob(details["atom_files"]))
            ):
                # Generate instance of IncidentPhoton
                print("Converting:", photo_path.name, atom_path.name)
                data = openmc.data.IncidentPhoton.from_endf(photo_path, atom_path)

                # Export HDF5 file
                h5_file = args.destination.joinpath(particle, data.name + ".h5")
                data.export_to_hdf5(h5_file, "w", libver=args.libver)

                # Register with library
                library.register_file(h5_file)

    # Write cross_sections.xml
    print("Writing ", args.destination / "cross_sections.xml")
    library.export_to_xml(args.destination / "cross_sections.xml")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
from pathlib import Path
import shutil

import numpy as np
import openmc.data


def library_in_list(lib, lib_list):
    """Returns whether the given nuclide library is present in the given list of libraries.

    Does this by checking whether libraries have the same type and material list.
    """
    for lib_in_list in lib_list:
        if isinstance(lib_in_list, type(lib)):
            if lib['type'] == lib_in_list['type'] \
               and np.array_equal(lib['materials'], lib_in_list['materials']):
                return True
    return False


description = """
Script to combine nuclide files from multiple OpenMC HDF5 libraries into a single library.
"""


class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter,
                      argparse.RawDescriptionHelpFormatter):
    pass


parser = argparse.ArgumentParser(
    description=description,
    formatter_class=CustomFormatter
)
parser.add_argument('-d', '--destination', type=Path, default=None,
                    help='Directory to output new library in')
parser.add_argument('-o', '--outputfilename', type=str, default='cross_sections.xml',
                    help='Output filename')
parser.add_argument('-l', '--libraries', type=Path,
                    help='List of data library .xml files to combine', nargs='+')
parser.set_defaults(copy=True)
args = parser.parse_args()


def main():

    read_libraries = []
    copy = True

    # Error check arguments
    if args.destination is None:
        copy = False
        args.destination = Path('.')
    else:
        if args.destination.exists():
            if not args.destination.is_dir():
                raise NotADirectoryError(f'Destination {args.destination.resolve()} should be a directory')
            if any(args.destination.iterdir()):
                raise OSError(f'Destination {args.destination.resolve()} is not empty')

    # Parse library .xml files
    if args.libraries is None:
        raise OSError('No input libraries specified')
    for lib_cross_sections_file in args.libraries:
        parsed_library = openmc.data.DataLibrary.from_xml(lib_cross_sections_file)
        read_libraries.append(parsed_library)

    print(f'Creating library {args.destination.resolve() / args.outputfilename}'
        ' from the following libraries in order of preference:')
    for i, lib_dir in enumerate(args.libraries):
        print(f'{i + 1}) {lib_dir.resolve()}')

    # Create output directory if it doesn't exist
    if copy:
        print('Original library files will be copied into the destination folder')
        args.destination.mkdir(parents=True, exist_ok=True)

    combined_library = openmc.data.DataLibrary()

    # Copy all of library 1 to new library
    for library in read_libraries[0].libraries:
        source_file = Path(library['path'])
        destination_file = source_file
        if copy:
            destination_file = args.destination / source_file.name
            shutil.copy(source_file, args.destination)
        print(f'Adding {source_file.name} from {args.libraries[0].resolve()}')
        combined_library.register_file(destination_file)

    # For each other libraries, check library and add if not already present
    for lib_num in range(1, len(read_libraries)):
        for library in read_libraries[lib_num].libraries:
            if not library_in_list(library, combined_library.libraries):
                source_file = Path(library['path'])
                destination_file = source_file
                if copy:
                    destination_file = args.destination / source_file.name
                    if destination_file.exists():
                        raise FileExistsError(f'Library file {destination_file.name} already'
                                            ' exists in the combined library')
                    shutil.copy(source_file, args.destination)
                print(f'Adding {source_file.name} from {args.libraries[lib_num].resolve()}')
                combined_library.register_file(destination_file)

    # Write .xml file
    combined_library_path = args.destination / args.outputfilename
    combined_library.export_to_xml(combined_library_path)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import argparse
import os
import tarfile
from pathlib import Path
from multiprocessing import Pool

import openmc.data

from openmc_data import download, state_download_size

description = """
Download random TENDL libraries from PSI and convert it to a HDF5 library for use with OpenMC. 
Only certain nuclides are available from PSI. This script generates a cross_sections_tendl.xml 
file with random TENDL evaluations plus a standard library located in 'OPENMC_CROSS_SECTIONS'
"""


class CustomFormatter(
    argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter
):
    pass


n_choices = [
    "all",
    "O16",
    "Si28",
    "Si29",
    "Si30",
    "Fe54",
    "Fe56",
    "Fe57",
    "Fe58",
    "Na23",
    "Pu240",
]

parser = argparse.ArgumentParser(
    description=description, formatter_class=CustomFormatter
)
parser.add_argument(
    "-n",
    "--nuclides",
    choices=n_choices,
    nargs="+",
    default=["Fe56"],
    help="The nuclides to be downloaded. Available are: "
    "'O16','Si28', 'Si29','Si30', 'Fe54', 'Fe56', 'Fe57', 'Fe58', 'Na23', 'Pu240'. Use 'all' for all availiable",
)
parser.add_argument(
    "-d", "--destination", default=None, help="Directory to create new library in"
)
parser.add_argument(
    "-x",
    "--xlib",
    default=None,
    help="cross_section.xml library to add random evaluations to",
)
parser.add_argument("-b", "--batch", action="store_true", help="supresses standard in")
parser.add_argument(
    "-f",
    "--format_only",
    default=False,
    help="Only format previously sampled files to HDF5",
)

args = parser.parse_args()


def process_neutron_random(nuc, i, out_dir, in_dir, file_num):
    """Process ENDF neutron sublibrary file into HDF5 and write into a
    specified output directory."""

    fileIn = in_dir / f"{nuc}-{i}"
    fileOut = out_dir / f"{nuc}-{i}.h5"

    data = openmc.data.IncidentNeutron.from_njoy(fileIn)
    data.name = f"{nuc}-{i}"
    data.export_to_hdf5(fileOut, "w")
    if i % 40 == 0:
        print(f"Nuclide {nuc} {i}/{file_num} finished")


def main():

    if "all" in args.nuclides:
        nuclides = [
            "O16",
            "Si28",
            "Si29",
            "Si30",
            "Fe54",
            "Fe56",
            "Fe57",
            "Fe58",
            "Na23",
            "Pu240",
        ]
    else:
        nuclides = args.nuclides
    script_dir = Path.cwd()

    library_name = "tendl_rand"  # this could be added as an argument to allow different libraries to be downloaded

    # the destination is decided after the release is know to avoid putting the release in a folder with a misleading name
    if args.destination is None:
        output_dir = script_dir / library_name
    else:
        output_dir = Path(args.destination).resolve()

    xlib = args.xlib
    if xlib == None:
        xlib = os.getenv("OPENMC_CROSS_SECTIONS")
    else:
        xlib = Path(xlib).resolve()

    endf_files_dir = output_dir / "endf"
    ace_files_dir = output_dir / "ace"
    hdf5_files_dir = output_dir / "hdf5"

    format_only = args.format_only

    release_details = {
        "ENDF2017": {
            "base_url": "https://tendl.web.psi.ch/tendl_2017/tar_files/",
            "ending": ".random.tgz",
        },
        "ENDF2015": {
            "base_url": "https://tendl.web.psi.ch/tendl_2015/tar_files/",
            "ending": ".random.tgz",
        },
        "ACE2015": {
            "base_url": "https://tendl.web.psi.ch/tendl_2015/tar_files/",
            "ending": ".10.2016.tgz",
        },
    }

    nuclide_details = {
        "O16": {
            "release": "ENDF2017",
            "filename": "O016",
            "webname": "O016",
            "file_num": 642,
            "down_size": 60,
            "file_size": 244,
            "is_it_ENDF": True,
            "gunzip": False,
        },
        "Si28": {
            "release": "ENDF2017",
            "filename": "Si028",
            "webname": "Si28",
            "file_num": 600,
            "down_size": 110,
            "file_size": 428,
            "is_it_ENDF": True,
            "gunzip": True,
        },
        "Si29": {
            "release": "ENDF2017",
            "filename": "Si029",
            "webname": "Si29",
            "file_num": 600,
            "down_size": 100,
            "file_size": 425,
            "is_it_ENDF": True,
            "gunzip": True,
        },
        "Si30": {
            "release": "ENDF2017",
            "filename": "Si030",
            "webname": "Si30",
            "file_num": 600,
            "down_size": 80,
            "file_size": 337,
            "is_it_ENDF": True,
            "gunzip": True,
        },
        "Fe54": {
            "release": "ENDF2017",
            "filename": "Fe054",
            "webname": "Fe054",
            "file_num": 501,
            "down_size": 320,
            "file_size": 1300,
            "is_it_ENDF": True,
            "gunzip": False,
        },
        "Fe56": {
            "release": "ENDF2017",
            "filename": "Fe056",
            "webname": "Fe056",
            "file_num": 614,
            "down_size": 410,
            "file_size": 1600,
            "is_it_ENDF": True,
            "gunzip": False,
        },
        "Fe57": {
            "release": "ENDF2017",
            "filename": "Fe057",
            "webname": "Fe057",
            "file_num": 733,
            "down_size": 525,
            "file_size": 2000,
            "is_it_ENDF": True,
            "gunzip": False,
        },
        "Fe58": {
            "release": "ENDF2017",
            "filename": "Fe058",
            "webname": "Fe058",
            "file_num": 501,
            "down_size": 280,
            "file_size": 1100,
            "is_it_ENDF": True,
            "gunzip": False,
        },
        "Na23": {
            "release": "ENDF2017",
            "filename": "Na023",
            "webname": "Na23",
            "file_num": 832,
            "down_size": 130,
            "file_size": 544,
            "is_it_ENDF": True,
            "gunzip": False,
        },
        "Pu240": {
            "release": "ENDF2017",
            "filename": "Pu240",
            "webname": "Pu240",
            "file_num": 630,
            "down_size": 200,
            "file_size": 900,
            "is_it_ENDF": True,
            "gunzip": False,
        },
    }

    if not format_only:

        download_file_size = 0
        uncompressed_file_size = 0
        num_of_files = 0

        for i in nuclides:
            download_file_size += nuclide_details[i]["down_size"]
            uncompressed_file_size += nuclide_details[i]["file_size"]
            num_of_files += nuclide_details[i]["file_num"]

        state_download_size(download_file_size, uncompressed_file_size, 'MB')
        extra_download_warning = f"""This corresponds to {num_of_files} random
            cross sections. The nuclides to be processed are: {nuclides}"""
        print(extra_download_warning)

        # ==============================================================================
        # DOWNLOAD FILES FROM WEBSITE

        files_complete = []
        for nucs in nuclides:
            # Establish connection to URL
            url = (
                release_details[nuclide_details[nucs]["release"]]["base_url"]
                + nuclide_details[nucs]["webname"]
                + release_details[nuclide_details[nucs]["release"]]["ending"]
            )
            print(f"Downloading {nucs}...")
            downloaded_file = download(url)

        # ==============================================================================
        # EXTRACT FILES FROM TGZ

        for nucs in nuclides:
            f = nuclide_details[nucs]["webname"] + ".random.tgz"
            suffix = nucs
            is_it_ENDF = nuclide_details[nucs]["is_it_ENDF"]
            out_dir = endf_files_dir if is_it_ENDF else ace_files_dir

            with tarfile.open(f, "r") as tgz:
                print(f"Extracting {f}...")
                tgz.extractall(path=out_dir / suffix)

        # ==============================================================================
        # Format file names

        for nucs in nuclides:
            f = nuclide_details[nucs]["webname"] + ".random.tgz"
            is_it_ENDF = nuclide_details[nucs]["is_it_ENDF"]
            numFiles = nuclide_details[nucs]["file_num"]

            if is_it_ENDF:
                out_dir = endf_files_dir / nucs
                prefix = "n-"
                suffix = "-rand-"

                for i in range(numFiles):
                    OldNumber = f"{i:04}"
                    OldFile = (
                        prefix + nuclide_details[nucs]["filename"] + suffix + OldNumber
                    )
                    newFile = f"{nucs}-{i+1}"

                    if nuclide_details[nucs]["gunzip"]:
                        os.system(f"gunzip {out_dir}/{OldFile}.gz")
                    (out_dir / OldFile).rename(out_dir / newFile)
            os.remove(f)

    # ==============================================================================
    # Convert ENDF files to HDF5 with njoy

    print("Beginning NJOY processing")
    with Pool() as pool:
        results = []
        for nuc in nuclides:

            file_num = nuclide_details[nuc]["file_num"]
            in_dir = endf_files_dir / nuc
            out_dir = hdf5_files_dir / nuc

            hdf5_files_dir.mkdir(exist_ok=True)
            out_dir.mkdir(exist_ok=True)

            print(f"Beginning nuclide {nuc} ...")
            for i in range(1, file_num + 1):
                func_args = (nuc, i, out_dir, in_dir, file_num)
                r = pool.apply_async(process_neutron_random, func_args)
                results.append(r)

        for r in results:
            r.wait()

    # ==============================================================================
    # Create xml library

    lib = openmc.data.DataLibrary()
    lib = lib.from_xml(xlib)  # Gets current

    for nuc in nuclides:
        file_num = nuclide_details[nuc]["file_num"]
        out_dir = hdf5_files_dir / nuc
        for i in range(1, file_num + 1):
            fileOut = out_dir / f"{nuc}-{i}.h5"
            lib.register_file(fileOut)

    pre = output_dir / "cross_sections_pre.xml"
    post = output_dir / "cross_sections_tendl.xml"

    lib.export_to_xml(pre)
    if post.exists():
        command = f"combine_libraries.py -l {pre} {post} -o {post}"
        os.system(command)
    else:
        lib.export_to_xml(post)

    pre.unlink()


if __name__ == "__main__":
    main()



all_release_details = {
    "tendl": {
        "2015": {
            "neutron": {
                "base_url": "https://tendl.web.psi.ch/tendl_2015/tar_files/",
                "compressed_files": ["ACE-n.tgz"],
                "neutron_files": "neutron_file/*/*/lib/endf/*-n.ace",
                "metastables": "neutron_file/*/*/lib/endf/*m-n.ace",
                "compressed_file_size": "5.1",
                "uncompressed_file_size": "40",
            }
        },
        "2017": {
            "neutron": {
                "base_url": "https://tendl.web.psi.ch/tendl_2017/tar_files/",
                "compressed_files": ["tendl17c.tar.bz2"],
                "neutron_files": "ace-17/*",
                "metastables": "ace-17/*m",
                "compressed_file_size": "2.1",
                "uncompressed_file_size": "14",
            }
        },
        "2019": {
            "neutron": {
                "base_url": "https://tendl.web.psi.ch/tendl_2019/tar_files/",
                "compressed_files": ["tendl19c.tar.bz2"],
                "neutron_files": "tendl19c/*",
                "metastables": "tendl19c/*m",
                "compressed_file_size": "2.3",
                "uncompressed_file_size": "10.1",
            }
        },
        "2021": {
            "neutron": {
                "base_url": "https://tendl.web.psi.ch/tendl_2021/tar_files/",
                "compressed_files": ["tendl21c.tar.bz2"],
                "neutron_files": "tendl21c/*",
                "metastables": "tendl21c/*m",
                "compressed_file_size": "2.2",
                "uncompressed_file_size": "10.5",
            }
        },
    },
    "fendl": {
        "3.2": {
            "neutron": {
                "base_url": "https://www-nds.iaea.org/fendl_library/websites/fendl32/data/neutron/",
                "compressed_files": ["fendl32-neutron-ace.zip"],
                "file_type": "ace",
                "ace_files": "neutron/ace/*[!.xsd]",
                "compressed_file_size": 565,
                "uncompressed_file_size": 4226,
            },
            "photon": {
                "base_url": "https://www-nds.iaea.org/fendl_library/websites/fendl32/data/atom/",
                "compressed_files": ["fendl32-atom-endf.zip"],
                "file_type": "endf",
                "endf_files": "atom/endf/*.endf",
                "compressed_file_size": 4,
                "uncompressed_file_size": 33,
            },
        },
        "3.1a": {
            "neutron": {
                "base_url": "https://www-nds.iaea.org/fendl31/data/neutron/",
                "compressed_files": ["fendl31a-neutron-ace.zip"],
                "file_type": "ace",
                "ace_files": "*.ace",
                "compressed_file_size": 384,
                "uncompressed_file_size": 2250,
            },
            "photon": {
                "base_url": "https://www-nds.iaea.org/fendl31/data/atom/",
                "compressed_files": ["fendl30-atom-endf.zip"],
                "file_type": "endf",
                "endf_files": "endf/*.txt",
                "compressed_file_size": 4,
                "uncompressed_file_size": 12,
            },
        },
        "3.1d": {
            "neutron": {
                "base_url": "https://www-nds.iaea.org/fendl31d/data/neutron/",
                "compressed_files": ["fendl31d-neutron-ace.zip"],
                "file_type": "ace",
                "ace_files": "fendl31d_ACE/*",
                "compressed_file_size": 425,
                "uncompressed_file_size": 2290,
            },
            "photon": {
                "base_url": "https://www-nds.iaea.org/fendl31d/data/atom/",
                "compressed_files": ["fendl30-atom-endf.zip"],
                "file_type": "endf",
                "endf_files": "endf/*.txt",
                "compressed_file_size": 4,
                "uncompressed_file_size": 12,
            },
        },
        "3.0": {
            "neutron": {
                "base_url": "https://www-nds.iaea.org/fendl30/data/neutron/",
                "compressed_files": ["fendl30-neutron-ace.zip"],
                "file_type": "ace",
                "ace_files": "ace/*.ace",
                "compressed_file_size": 364,
                "uncompressed_file_size": 2200,
            },
            "photon": {
                "base_url": "https://www-nds.iaea.org/fendl30/data/atom/",
                "compressed_files": ["fendl30-atom-endf.zip"],
                "file_type": "endf",
                "endf_files": "endf/*.txt",
                "compressed_file_size": 4,
                "uncompressed_file_size": 12,
            },
        },
        "2.1": {
            "neutron": {
                "base_url": "https://www-nds.iaea.org/fendl21/fendl21mc/",
                "compressed_files": [
                    "H001mc.zip",
                    "H002mc.zip",
                    "H003mc.zip",
                    "He003mc.zip",
                    "He004mc.zip",
                    "Li006mc.zip",
                    "Li007mc.zip",
                    "Be009mc.zip",
                    "B010mc.zip",
                    "B011mc.zip",
                    "C012mc.zip",
                    "N014mc.zip",
                    "N015mc.zip",
                    "O016mc.zip",
                    "F019mc.zip",
                    "Na023mc.zip",
                    "Mg000mc.zip",
                    "Al027mc.zip",
                    "Si028mc.zip",
                    "Si029mc.zip",
                    "Si030mc.zip",
                    "P031mc.zip",
                    "S000mc.zip",
                    "Cl035mc.zip",
                    "Cl037mc.zip",
                    "K000mc.zip",
                    "Ca000mc.zip",
                    "Ti046mc.zip",
                    "Ti047mc.zip",
                    "Ti048mc.zip",
                    "Ti049mc.zip",
                    "Ti050mc.zip",
                    "V000mc.zip",
                    "Cr050mc.zip",
                    "Cr052mc.zip",
                    "Cr053mc.zip",
                    "Cr054mc.zip",
                    "Mn055mc.zip",
                    "Fe054mc.zip",
                    "Fe056mc.zip",
                    "Fe057mc.zip",
                    "Fe058mc.zip",
                    "Co059mc.zip",
                    "Ni058mc.zip",
                    "Ni060mc.zip",
                    "Ni061mc.zip",
                    "Ni062mc.zip",
                    "Ni064mc.zip",
                    "Cu063mc.zip",
                    "Cu065mc.zip",
                    "Ga000mc.zip",
                    "Zr000mc.zip",
                    "Nb093mc.zip",
                    "Mo092mc.zip",
                    "Mo094mc.zip",
                    "Mo095mc.zip",
                    "Mo096mc.zip",
                    "Mo097mc.zip",
                    "Mo098mc.zip",
                    "Mo100mc.zip",
                    "Sn000mc.zip",
                    "Ta181mc.zip",
                    "W182mc.zip",
                    "W183mc.zip",
                    "W184mc.zip",
                    "W186mc.zip",
                    "Au197mc.zip",
                    "Pb206mc.zip",
                    "Pb207mc.zip",
                    "Pb208mc.zip",
                    "Bi209mc.zip",
                ],
                "file_type": "ace",
                "ace_files": "*.ace",
                "compressed_file_size": 100,
                "uncompressed_file_size": 600,
            },
            "photon": {
                "base_url": "https://www-nds.iaea.org/fendl21/fendl21e/",
                "compressed_files": ["FENDLEP.zip"],
                "file_type": "endf",
                "endf_files": "*.endf",
                "compressed_file_size": 2,
                "uncompressed_file_size": 5,
            },
        },
    },
    'nndc': {
        "b7.1": {
            "neutron": {
                "base_url": "http://www.nndc.bnl.gov/endf-b7.1/aceFiles/",
                "compressed_files": [
                    "ENDF-B-VII.1-neutron-293.6K.tar.gz",
                    "ENDF-B-VII.1-tsl.tar.gz",
                ],
                "checksums": [
                    "9729a17eb62b75f285d8a7628ace1449",
                    "e17d827c92940a30f22f096d910ea186",
                ],
                "file_type": "ace",
                "ace_files": "[aA-zZ]*.ace",
                "sab_files": "*.acer",
                "compressed_file_size": 497,
                "uncompressed_file_size": 1200,
            },
            "photon": {
                "base_url": "http://www.nndc.bnl.gov/endf-b7.1/zips/",
                "compressed_files": [
                    "ENDF-B-VII.1-photoat.zip",
                    "ENDF-B-VII.1-atomic_relax.zip",
                ],
                "checksums": [
                    "5192f94e61f0b385cf536f448ffab4a4",
                    "fddb6035e7f2b6931e51a58fc754bd10",
                ],
                "file_type": "endf",
                "photo_files": "photoat/*.endf",
                "atom_files": "atomic_relax/*.endf",
                "compressed_file_size": 9,
                "uncompressed_file_size": 45,
            },
        }
    },
    'jeff': {
        "3.2": {
            'neutron': {
                "base_url": "https://www.oecd-nea.org/dbforms/data/eva/evatapes/jeff_32/Processed/",
                "compressed_files": [
                    "JEFF32-ACE-293K.tar.gz",
                    "JEFF32-ACE-400K.tar.gz",
                    "JEFF32-ACE-500K.tar.gz",
                    "JEFF32-ACE-600K.tar.gz",
                    "JEFF32-ACE-700K.tar.gz",
                    "JEFF32-ACE-800K.zip",  # Note the different suffix
                    "JEFF32-ACE-900K.tar.gz",
                    "JEFF32-ACE-1000K.tar.gz",
                    "JEFF32-ACE-1200K.tar.gz",
                    "JEFF32-ACE-1500K.tar.gz",
                    "JEFF32-ACE-1800K.tar.gz",
                    "TSLs.tar.gz"],
                "temperatures": [
                    "293",
                    "400",
                    "500",
                    "600",
                    "700",
                    "800",
                    "900",
                    "1000",
                    "1200",
                    "1500",
                    "1800",
                    None
                ],
                "neutron_files": "*.ACE",
                "metastables": "*M.ACE",
                "sab_files": "ANNEX_6_3_STLs/*/*.ace",
                "redundant": "ACEs_293K/*-293.ACE",
                "compressed_file_size": 9,
                "uncompressed_file_size": 40,
            }
        },
        "3.3": {
            'neutron': {
                "base_url": "http://www.oecd-nea.org/dbdata/jeff/jeff33/downloads/temperatures/",
                "compressed_files": [
                    "ace_293.tar.gz",
                    "ace_600.tar.gz",
                    "ace_900.tar.gz",
                    "ace_1200.tar.gz",
                    "ace_1500.tar.gz",
                    "ace_1800.tar.gz",
                    "ace_tsl.tar.gz",
                ],
                "temperatures": [
                    "293",
                    "600",
                    "900",
                    "1200",
                    "1500",
                    "1800",
                    None
                ],
                "neutron_files": "ace_293/*.ace",
                "thermal_files": "ace_tsl",
                "compressed_file_size": "7.7",
                "uncompressed_file_size": "37",
            }
        }
    },
    'jendl': {
        '4.0': {
            'neutron': {
                'base_url': 'https://wwwndc.jaea.go.jp/ftpnd/ftp/JENDL/',
                'compressed_files': ['jendl40-or-up_20160106.tar.gz'],
                'endf_files': 'jendl40-or-up_20160106/*.dat',
                'metastables': 'jendl40-or-up_20160106/*m.dat',
                'compressed_file_size': '0.2',
                'uncompressed_file_size': '2'
            }
        }
    }
}



#!/usr/bin/env python3

"""
Downloads preprocessed a chain file in xml format for use with OpenMc depletion
simulations.
"""

import argparse
from pathlib import Path
from urllib.parse import urljoin

from openmc_data.urls_xml import all_chain_release_details
from openmc_data.utils import download


class CustomFormatter(
    argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter
):
    pass


parser = argparse.ArgumentParser(description=__doc__, formatter_class=CustomFormatter)
parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default=None,
    help="Directory to create new library in",
)
parser.add_argument(
    "-f",
    "--filename",
    type=Path,
    default=None,
    help="Filename for the xml chain file",
)
parser.add_argument(
    "-r",
    "--release",
    choices=["b7.1"],
    default="b7.1",
    help="The nuclear data library release version. The currently supported "
         "options are b7.1",
)

parser.set_defaults()
args = parser.parse_args()


def main():

    library_name = 'nndc'
    details = all_chain_release_details[library_name][args.release]["chain"]

    cwd = Path.cwd()

    if args.filename is None:
        args.filename = Path("-".join(["chain", library_name, args.release])+".xml")
    print(args.filename)

    download(
        details["url"],
        output_path=args.destination,
        output_filename=args.filename,
    )


if __name__ == "__main__":
    main()



import hashlib
import imp
import tarfile
import warnings
import zipfile
from pathlib import Path
from shutil import rmtree
from urllib.parse import urlparse
from urllib.request import Request, urlopen
import warnings

import openmc.data

_BLOCK_SIZE = 16384


def state_download_size(download_size, uncompressed_size, units):
    """Prints a standard message to users displaying the amount of storage
    space required to run the script"""

    msg = (f"WARNING: This script will download up to {download_size} {units} "
           "of data. Extracting and processing the data may require as much "
           f"as {uncompressed_size} {units} of additional free disk space.")
    warnings.warn(msg)


def process_neutron(path, output_dir, libver, temperatures=None):
    """Process ENDF neutron sublibrary file into HDF5 and write into a
    specified output directory."""
    print(f'Converting: {path}')
    try:
        with warnings.catch_warnings():
            warnings.simplefilter('ignore', UserWarning)
            data = openmc.data.IncidentNeutron.from_njoy(
                path, temperatures=temperatures
            )
    except Exception as e:
        print(path, e)
        raise
    h5_file = output_dir / f'{data.name}.h5'
    print(f'Writing {h5_file} ...')
    data.export_to_hdf5(h5_file, 'w', libver=libver)


def process_thermal(path_neutron, path_thermal, output_dir, libver):
    """Process ENDF thermal scattering sublibrary file into HDF5 and write into a
    specified output directory."""
    print(f'Converting: {path_thermal}')
    try:
        with warnings.catch_warnings():
            warnings.simplefilter('ignore', UserWarning)
            data = openmc.data.ThermalScattering.from_njoy(
                path_neutron, path_thermal
            )
    except Exception as e:
        print(path_neutron, path_thermal, e)
        raise
    h5_file = output_dir / f'{data.name}.h5'
    print(f'Writing {h5_file} ...')
    data.export_to_hdf5(h5_file, 'w', libver=libver)


def extract(
    compressed_files,
    extraction_dir,
    del_compressed_file=False,
    verbose=True,
):
    """Extracts zip, tar.gz or tgz compressed files

    Parameters
    ----------
    compressed_files : iterable
        The files to extract.
    extraction_dir : str
        The directory to extract the files to.
    del_compressed_file : bool
        Wheather the compressed file should be deleted (True) or not (False)
    verbose : bool
        Controls the printing to terminal, if True filenames of the extracted
        files will be printed.
    """
    Path.mkdir(extraction_dir, parents=True, exist_ok=True)

    for f in compressed_files:
        if str(f).endswith('.zip'):
            with zipfile.ZipFile(f, 'r') as zipf:
                if verbose:
                    print(f'Extracting {f}...')
                zipf.extractall(path=extraction_dir)

        elif str(f).endswith('.tar.gz') or str(f).endswith('.tgz') or str(f).endswith('.tar.bz2') or str(f).endswith('.tar.xz')  or str(f).endswith('.xz'):
            with tarfile.open(f, 'r') as tgz:
                if verbose:
                    print(f'Extracting {f}...')
                tgz.extractall(path=extraction_dir)

        else:
            raise ValueError('File type not currently supported by extraction '
                             f'function {str(f)}')

    if del_compressed_file:
        rmtree(compressed_files, ignore_errors=True)


def download(url, checksum=None, as_browser=False, output_path=None, output_filename=None, **kwargs):
    """Download file from a URL

    Parameters
    ----------
    url : str
        URL from which to download
    checksum : str or None
        MD5 checksum to check against
    as_browser : bool
        Change User-Agent header to appear as a browser
    output_path : str or Path
        Specifies the directory location to save the downloaded file
    output_filename : str or Path
        Specifies the filename save the downloaded file. If left as None the
        filename of the download file is obtained from the url filename
    kwargs : dict
        Keyword arguments passed to :func:`urllib.request.urlopen`

    Returns
    -------
    local_path : pathlib.Path
        Name of file written locally

    """
    if as_browser:
        page = Request(url, headers={'User-Agent': 'Mozilla/5.0'})
    else:
        page = url

    with urlopen(page, **kwargs) as response:
        # Get file size from header
        file_size = response.length

        if output_filename is None:
            local_path = Path(Path(urlparse(url).path).name)
        else:
            if output_path is None:
                local_path = Path(output_filename)
            else:
                local_path = Path(output_path)/Path(output_filename)

        if output_path is not None:
            Path(output_path).mkdir(parents=True, exist_ok=True)
            local_path = output_path / local_path
        # Check if file already downloaded
        if local_path.is_file():
            if local_path.stat().st_size == file_size:
                print('Skipping {}, already downloaded'.format(local_path))
                return local_path

        # Copy file to disk in chunks
        print(f'Downloading {local_path}... ', end='')
        downloaded = 0
        with open(local_path, 'wb') as fh:
            while True:
                chunk = response.read(_BLOCK_SIZE)
                if not chunk:
                    break
                fh.write(chunk)
                downloaded += len(chunk)
                status = '{:10}  [{:3.2f}%]'.format(
                    downloaded, downloaded * 100. / file_size)
                print(status + '\b'*len(status), end='', flush=True)
            print('')

    if checksum is not None:
        downloadsum = hashlib.md5(open(local_path, 'rb').read()).hexdigest()
        if downloadsum != checksum:
            raise OSError("MD5 checksum for {} does not match. If this is "
                          "your first time receiving this message, please "
                          "re-run the script. Otherwise, please contact "
                          "OpenMC developers by emailing "
                          "openmc-users@googlegroups.com.".format(local_path))

    return local_path




import requests

from openmc_data import all_release_details, all_h5_release_details, all_chain_release_details


def test_convert_urls():
    """Cycles through all the urls in each nuclear data library and checks
    that they return a status 200 code (success)"""

    print("library, release, particle, responce.status_code")
    for library, releases in all_release_details.items():
        for release, particles in releases.items():
            for particle, value in particles.items():
                for file in value['compressed_files']:
                    url = value['base_url'] + file
                    print(library, release, particle, url)
                    responce = requests.get(url, stream=True)
                    print(library, release, particle, url, responce.status_code)
                    # printing output so that in the event of a failure the
                    # failing url can be identified
                    assert responce.status_code == 200


def test_h5_urls():
    """Cycles through all the h5 urls in each nuclear data library and checks
    that they return a status 200 code (success)"""

    print("library, release, particle, responce.status_code")
    for library, releases in all_h5_release_details.items():
        for release, particles in releases.items():
            for particle, value in particles.items():
                for file in value['compressed_files']:
                    url = value['base_url'] + file
                    print(library, release, particle, url)
                    responce = requests.get(url, stream=True)
                    print(library, release, particle, url, responce.status_code)
                    # printing output so that in the event of a failure the
                    # failing url can be identified
                    assert responce.status_code == 200


def test_xml_urls():
    """Cycles through all the xml urls in each nuclear data library and checks
    that they return a status 200 code (success)"""

    print("library, release, particle, responce.status_code")
    for library, releases in all_chain_release_details.items():
        for release, chains in releases.items():
            url = chains['chain']['url']
            print(library, release, url)
            responce = requests.get(url, stream=True)
            print(library, release, url, responce.status_code)
            # printing output so that in the event of a failure the
            # failing url can be identified
            assert responce.status_code == 200



#!/usr/bin/env python3

"""
Download JEFF 3.2 ACE data from OECD/NEA and convert it to a multi-temperature
HDF5 library for use with OpenMC.
"""

import argparse
import tarfile
import zipfile
from collections import defaultdict
from pathlib import Path
from string import digits
from urllib.parse import urljoin

import openmc.data
from openmc_data import download, state_download_size, all_release_details


class CustomFormatter(
    argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter
):
    pass


parser = argparse.ArgumentParser(description=__doc__, formatter_class=CustomFormatter)
parser.add_argument(
    "-d",
    "--destination",
    type=Path,
    default=None,
    help="Directory to create new library in",
)
parser.add_argument(
    "--download", action="store_true", help="Download files from OECD-NEA"
)
parser.add_argument(
    "--no-download",
    dest="download",
    action="store_false",
    help="Do not download files from OECD-NEA",
)
parser.add_argument("--extract", action="store_true", help="Extract tar/zip files")
parser.add_argument(
    "--no-extract",
    dest="extract",
    action="store_false",
    help="Do not extract tar/zip files",
)
parser.add_argument(
    "--libver",
    choices=["earliest", "latest"],
    default="latest",
    help="Output HDF5 versioning. Use "
    "'earliest' for backwards compatibility or 'latest' for "
    "performance",
)
parser.add_argument(
    "-r",
    "--release",
    choices=["3.2"],
    default="3.2",
    help="The nuclear data library release version. "
    "The currently supported options are 3.2",
)
parser.add_argument(
    "-t",
    "--temperatures",
    choices=[
        "293",
        "400",
        "500",
        "600",
        "700",
        "800",
        "900",
        "1000",
        "1200",
        "1500",
        "1800",
    ],
    default=[
        "293",
        "400",
        "500",
        "600",
        "700",
        "800",
        "900",
        "1000",
        "1200",
        "1500",
        "1800",
    ],
    help="Temperatures to download in Kelvin",
    nargs="+",
)
parser.add_argument(
    "--cleanup",
    action="store_true",
    help="Remove download directories when data has " "been processed",
)
parser.add_argument(
    "--no-cleanup",
    dest="cleanup",
    action="store_false",
    help="Do not remove download directories when data has " "been processed",
)
parser.set_defaults(download=True, extract=True, cleanup=False)
args = parser.parse_args()


def main():

    library_name = "jeff"

    cwd = Path.cwd()

    ace_files_dir = cwd.joinpath("-".join([library_name, args.release, "ace"]))
    download_path = cwd.joinpath("-".join([library_name, args.release, "download"]))
    # the destination is decided after the release is know to avoid putting the release in a folder with a misleading name
    if args.destination is None:
        args.destination = Path("-".join([library_name, args.release, "hdf5"]))

    # This dictionary contains all the unique information about each release. This can be exstened to accommodated new releases
    details = all_release_details[library_name][args.release]['neutron']

    # ==============================================================================
    # DOWNLOAD FILES FROM OECD SITE

    if args.download:
        state_download_size(details["compressed_file_size"], details["uncompressed_file_size"], 'GB')
        for f, t in zip(details["compressed_files"], details["temperatures"]):
            if t in args.temperatures or t is None:
                download(
                    urljoin(details["base_url"], f),
                    output_path=download_path,
                )

    # ==============================================================================
    # EXTRACT FILES FROM TGZ

    if args.extract:
        for f, t in zip(details["compressed_files"], details["temperatures"]):
            if t in args.temperatures or t is None:
                # Extract files
                if f.endswith(".zip"):
                    with zipfile.ZipFile(download_path / f, "r") as zipf:
                        print("Extracting {}...".format(f))
                        zipf.extractall(ace_files_dir)

                else:
                    suffix = "ACEs_293K" if t == "293" else ""
                    with tarfile.open(download_path / f, "r") as tgz:
                        print("Extracting {}...".format(f))
                        tgz.extractall(ace_files_dir / suffix)

                    # Remove thermal scattering tables from 293K data since they
                    # are redundant
                    if t == "293":
                        redundants = ace_files_dir.glob(details["redundant"])
                        for path in redundants:
                            print(f"removing {path}")
                            path.unlink()

    # ==============================================================================
    # CHANGE ZAID FOR METASTABLES

    metastables = ace_files_dir.rglob(details["metastables"])
    for path in metastables:
        print("    Fixing {} (ensure metastable)...".format(path))
        text = open(path, "r").read()
        mass_first_digit = int(text[3])
        if mass_first_digit <= 2:
            text = text[:3] + str(mass_first_digit + 4) + text[4:]
            open(path, "w").write(text)

    # ==============================================================================
    # GENERATE HDF5 LIBRARY -- NEUTRON FILES

    # Get a list of all ACE files
    neutron_files = ace_files_dir.rglob(details["neutron_files"])

    # Group together tables for same nuclide
    tables = defaultdict(list)
    for filename in sorted(neutron_files):
        name = filename.stem
        tables[name].append(filename)

    # Sort temperatures from lowest to highest
    for name, filenames in sorted(tables.items()):
        filenames.sort(key=lambda x: int(x.parts[-2].split("_")[1][:-1]))

    # Create output directory if it doesn't exist
    args.destination.mkdir(parents=True, exist_ok=True)

    library = openmc.data.DataLibrary()

    for name, filenames in sorted(tables.items()):
        # Convert first temperature for the table
        print("Converting: " + str(filenames[0]))
        data = openmc.data.IncidentNeutron.from_ace(filenames[0])

        # For each higher temperature, add cross sections to the existing table
        for filename in filenames[1:]:
            print(f"Adding: {filename}")
            data.add_temperature_from_ace(filename)

        # Export HDF5 file
        h5_file = args.destination / f"{data.name}.h5"
        print("Writing {}...".format(h5_file))
        data.export_to_hdf5(h5_file, "w", libver=args.libver)

        # Register with library
        library.register_file(h5_file)

    # ==============================================================================
    # GENERATE HDF5 LIBRARY -- S(A,B) FILES

    # Group together tables for same nuclide
    tables = defaultdict(list)
    sab_files = ace_files_dir.glob(details["sab_files"])
    for filename in sorted(sab_files):
        name = filename.name.split("-")[0]
        tables[name].append(filename)

    # Sort temperatures from lowest to highest
    for name, filenames in sorted(tables.items()):
        filenames.sort(key=lambda x: int(x.name.split("-")[1].split(".")[0]))

    for name, filenames in sorted(tables.items()):
        # Convert first temperature for the table
        print(f"Converting: {filenames[0]}")

        # Take numbers out of table name, e.g. lw10.32t -> lw.32t
        table = openmc.data.ace.get_table(filenames[0])
        name, xs = table.name.split(".")
        table.name = ".".join((name.strip(digits), xs))
        data = openmc.data.ThermalScattering.from_ace(table)

        # For each higher temperature, add cross sections to the existing table
        for filename in filenames[1:]:
            print(f"Adding: {filename}")
            table = openmc.data.ace.get_table(filename)
            name, xs = table.name.split(".")
            table.name = ".".join((name.strip(digits), xs))
            data.add_temperature_from_ace(table)

        # Export HDF5 file
        h5_file = args.destination / f"{data.name}.h5"
        print("Writing {}...".format(h5_file))
        data.export_to_hdf5(h5_file, "w", libver=args.libver)

        # Register with library
        library.register_file(h5_file)

    # Write cross_sections.xml
    library.export_to_xml(args.destination / "cross_sections.xml")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
Download JENDL 4.0 data from JAEA and convert it to a HDF5 library for
use with OpenMC.
"""

import argparse
import ssl
from multiprocessing import Pool
from pathlib import Path
from urllib.parse import urljoin

import openmc.data
from openmc_data import download, extract, process_neutron, state_download_size, all_release_details


class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter,
                      argparse.RawDescriptionHelpFormatter):
    pass


parser = argparse.ArgumentParser(
    description=__doc__,
    formatter_class=CustomFormatter
)
parser.add_argument('-d', '--destination', type=Path, default=None,
                    help='Directory to create new library in')
parser.add_argument('--download', action='store_true',
                    help='Download files from JAEA')
parser.add_argument('--no-download', dest='download', action='store_false',
                    help='Do not download files from JAEA')
parser.add_argument('--extract', action='store_true',
                    help='Extract tar/zip files')
parser.add_argument('--no-extract', dest='extract', action='store_false',
                    help='Do not extract tar/zip files')
parser.add_argument('--libver', choices=['earliest', 'latest'],
                    default='latest', help="Output HDF5 versioning. Use "
                    "'earliest' for backwards compatibility or 'latest' for "
                    "performance")
parser.add_argument('-r', '--release', choices=['4.0'], default='4.0',
                    help="The nuclear data library release version. "
                    "The only option currently supported is 4.0")
parser.add_argument('--cleanup', action='store_true',
                    help="Remove download directories when data has "
                    "been processed")
parser.add_argument('--no-cleanup', dest='cleanup', action='store_false',
                    help="Do not remove download directories when data has "
                    "been processed")
parser.set_defaults(download=True, extract=True, cleanup=False)
args = parser.parse_args()


def main():

    library_name = 'jendl'

    cwd = Path.cwd()

    endf_files_dir = cwd.joinpath('-'.join([library_name, args.release, 'endf']))
    download_path = cwd.joinpath('-'.join([library_name, args.release, 'download']))
    # the destination is decided after the release is known
    # to avoid putting the release in a folder with a misleading name
    if args.destination is None:
        args.destination = Path('-'.join([library_name, args.release, 'hdf5']))

    # This dictionary contains all the unique information about each release.
    # This can be exstened to accommodated new releases
    details = all_release_details[library_name][args.release]['neutron']

    # ==============================================================================
    # DOWNLOAD FILES FROM WEBSITE

    if args.download:
        state_download_size(details['compressed_file_size'], details['uncompressed_file_size'], 'GB')
        for f in details['compressed_files']:
            # Establish connection to URL
            download(urljoin(details['base_url'], f),
                    context=ssl._create_unverified_context(),
                    output_path=download_path)

    # ==============================================================================
    # EXTRACT FILES FROM TGZ
    if args.extract:
        extract(
            compressed_files=[download_path/ f for f in details['compressed_files']],
            extraction_dir=endf_files_dir,
            del_compressed_file=args.cleanup
        )


    # ==============================================================================
    # GENERATE HDF5 LIBRARY -- NEUTRON FILES

    # Get a list of all ENDF files
    neutron_files = endf_files_dir.glob(details['endf_files'])

    # Create output directory if it doesn't exist
    args.destination.mkdir(parents=True, exist_ok=True)

    library = openmc.data.DataLibrary()


    with Pool() as pool:
        results = []
        for filename in sorted(neutron_files):
            func_args = (filename, args.destination, args.libver)
            r = pool.apply_async(process_neutron, func_args)
            results.append(r)

        for r in results:
            r.wait()

    # Register with library
    for p in sorted((args.destination).glob('*.h5')):
        library.register_file(p)

    # Write cross_sections.xml
    library.export_to_xml(args.destination / 'cross_sections.xml')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""
Convert ENDF/B-VII.0 ACE data from the MCNP5/6 distribution into an HDF5 library
that can be used by OpenMC. This assumes that you have a directory containing
files named endf70[a-k] and endf70sab. Optionally, if a recent photoatomic
library (e.g., eprdata14) is available, it can also be converted using the
--photon argument.
"""

import argparse
from collections import defaultdict
from pathlib import Path
import sys

import openmc.data


# Make sure Python version is sufficient
assert sys.version_info >= (3, 6), "Python 3.6+ is required"



class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter,
                      argparse.RawDescriptionHelpFormatter):
    pass


parser = argparse.ArgumentParser(
    description=__doc__,
    formatter_class=CustomFormatter
)
parser.add_argument('-d', '--destination', type=Path, default=Path('mcnp_endfb70'),
                    help='Directory to create new library in')
parser.add_argument('--libver', choices=['earliest', 'latest'],
                    default='earliest', help="Output HDF5 versioning. Use "
                    "'earliest' for backwards compatibility or 'latest' for "
                    "performance")
parser.add_argument('-p', '--photon', type=Path,
                    help='Path to photoatomic data library (eprdata12 or later)')
parser.add_argument('mcnpdata', type=Path,
                    help="Directory containing endf70[a-k], endf70sab, and mcplib")
args = parser.parse_args()


def main():

    # Check arguments to make sure they're valid
    assert args.mcnpdata.is_dir(), 'mcnpdata argument must be a directory'
    if args.photon is not None:
        assert args.photon.is_file(), 'photon argument must be an existing file'

    # Get a list of all neutron ACE files
    endf70 = args.mcnpdata.glob('endf70[a-k]')

    # Create output directory if it doesn't exist
    (args.destination / 'photon').mkdir(parents=True, exist_ok=True)

    library = openmc.data.DataLibrary()

    for path in sorted(endf70):
        print(f'Loading data from {path}...')
        lib = openmc.data.ace.Library(path)

        # Group together tables for the same nuclide
        tables = defaultdict(list)
        for table in lib.tables:
            zaid, xs = table.name.split('.')
            tables[zaid].append(table)

        for zaid, tables in sorted(tables.items()):
            # Convert first temperature for the table
            print(f'Converting: {tables[0].name}')
            data = openmc.data.IncidentNeutron.from_ace(tables[0], 'mcnp')

            # For each higher temperature, add cross sections to the existing table
            for table in tables[1:]:
                print(f'Adding: {table.name}')
                data.add_temperature_from_ace(table, 'mcnp')

            # Export HDF5 file
            h5_file = args.destination / f'{data.name}.h5'
            print(f'Writing {h5_file}...')
            data.export_to_hdf5(h5_file, 'w', libver=args.libver)

            # Register with library
            library.register_file(h5_file)

    # Handle S(a,b) tables
    endf70sab = args.mcnpdata / 'endf70sab'
    if endf70sab.exists():
        lib = openmc.data.ace.Library(endf70sab)

        # Group together tables for the same nuclide
        tables = defaultdict(list)
        for table in lib.tables:
            name, xs = table.name.split('.')
            tables[name].append(table)

        for zaid, tables in sorted(tables.items()):
            # Convert first temperature for the table
            print(f'Converting: {tables[0].name}')
            data = openmc.data.ThermalScattering.from_ace(tables[0])

            # For each higher temperature, add cross sections to the existing table
            for table in tables[1:]:
                print(f'Adding: {table.name}')
                data.add_temperature_from_ace(table)

            # Export HDF5 file
            h5_file = args.destination / f'{data.name}.h5'
            print(f'Writing {h5_file}...')
            data.export_to_hdf5(h5_file, 'w', libver=args.libver)

            # Register with library
            library.register_file(h5_file)

    # Handle photoatomic data
    if args.photon is not None:
        lib = openmc.data.ace.Library(args.photon)

        for table in lib.tables:
            # Convert first temperature for the table
            print(f'Converting: {table.name}')
            data = openmc.data.IncidentPhoton.from_ace(table)

            # Export HDF5 file
            h5_file = args.destination / 'photon' / f'{data.name}.h5'
            print(f'Writing {h5_file}...')
            data.export_to_hdf5(h5_file, 'w', libver=args.libver)

            # Register with library
            library.register_file(h5_file)

    # Write cross_sections.xml
    library.export_to_xml(args.destination / 'cross_sections.xml')


if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""
Writes a depletion chain XML file from a depletion chain XML file and branching
ratios
"""

import argparse
import json

import openmc.deplete

class CustomFormatter(
    argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter
):
    pass

parser = argparse.ArgumentParser(description=__doc__, formatter_class=CustomFormatter)
parser.add_argument('chain_in')
parser.add_argument('branching_ratios')
parser.add_argument('chain_out')
args = parser.parse_args()


def main():

    # Load existing chain
    chain = openmc.deplete.Chain.from_xml(args.chain_in)

    # Set branching ratios
    with open(args.branching_ratios) as fh:
        br = json.load(fh)
    chain.set_branch_ratios(br, strict=False)

    # Export to new XML file
    chain.export_to_xml(args.chain_out)


if __name__ == "__main__":
    main()
